package controles;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.Vector;

import javax.swing.ImageIcon;

import elements.Joueur;
import elements.Ennemis.Ennemi;
import elements.Explosions.Explosion;
import elements.Tirs.Bombe;
import elements.Tirs.Laser;
import elements.Tirs.Tir;
import elements.Tirs.TirEnnemi;
import elements.items.Item;
import elements.items.ItemTir;
import elements.items.ItemTirMP;
import menus.MenuAskScore;

/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class Environnement
{
	private static final long serialVersionUID = 1L;
	
	// MACROS
	public static int MODE_JEU 			= 0;
	public static int MODE_PAUSE 		= 1;
	public static int MODE_MENU_QUITTER = 2;
	public static int RETOUR_AU_MENU 	= 4;
	public static int RETOUR_AU_JEU 	= 5;
	public static int DEMANDER_NOM 		= 6;

	public static int INTRO 			= 100;	
	public static int YOU_LOOSE			= 101;	
	public static int LEVEL_CLEAR 		= 102;
	
	// Vectors
	public static Environnement env;

	// DONNEES
	public 	int 				gameState;
	public 	int 				pause;
	public 	int 				score;
	public 	int 				choix;
	public 	int 				invincible;
	public 	int 				blink;
	public 	int 				antiAliasing;
	public 	int 				antiAliasingText;
	public 	int 				effects;
	public  boolean				modeBoss;
	public 	boolean 			vivant;	
	public 	boolean 			jeuFini;
	public  boolean				inGame;
	public 	Clavier 			clavier;
	private String 				meilleurScore;
	
	// VIES CONTINUES ET BOMBES
	public 	int 				nombreDeContinues;
	public 	int 				nombreDeVies;
	public 	int 				nombreDeBombes;
	public 	int 				tempsAvantProchaineBombe;
	public  int 				LVL;
	public  int					difficulte;

	// Images
	private ImageIcon 			vie;	
	private ImageIcon 			bombs;		
	private ImageIcon 			imgPause;	
	private ImageIcon 			board;
	private ImageIcon[] 		retournerAuMenu;
	
	// Durees du menu
	private long 				boucle;
	private long 				dernierDeplacement;
	public  long 				tempsMort;
	private MenuAskScore 		menuAskScore;
	
	//
	public Vector<Tir> 			tirsJoueur;
	public Vector<TirEnnemi> 	tirsEnnemi;
	public Vector<Bombe>		bombes;
	public Vector<Ennemi> 		ennemis;
	public Vector<Item>			items;
	public Vector<Explosion> 	exps;
	public EnnemisManager 		em;
	public Laser				laser;
	
	public Joueur 			joueur;
	
	private Environnement()
	{
		vie					= new ImageIcon( Toolkit.getDefaultToolkit().getImage("Images/Board/vies.png") );
		bombs				= new ImageIcon( Toolkit.getDefaultToolkit().getImage("Images/Board/bombs.png") );		
		imgPause 			= new ImageIcon( Toolkit.getDefaultToolkit().getImage("Images/Menus/pause.png") );
		board				= new ImageIcon( Toolkit.getDefaultToolkit().getImage("Images/Board/board.png") );
		retournerAuMenu 	= new ImageIcon[2];
			
		retournerAuMenu[0] 	= new ImageIcon( Toolkit.getDefaultToolkit().getImage("Images/Menus/continuer.png") );
		retournerAuMenu[1] 	= new ImageIcon( Toolkit.getDefaultToolkit().getImage("Images/Menus/retournerAuMenu.png") );
		
		dernierDeplacement 	= 0;

		clavier 			= Clavier.getClavier();
		gameState 			= 0;
		pause 				= 0;
		score 				= 0;
		choix 				= 0;
		vivant 				= true;
		inGame				= false;
		
		nombreDeVies 		= 3;
		nombreDeBombes 		= 3;
		
		LVL					= 0;
		
		tirsJoueur 	= new Vector<Tir>(0);
		bombes 		= new Vector<Bombe>(0);		
		tirsEnnemi 	= new Vector<TirEnnemi>(0);	
		ennemis 	= new Vector<Ennemi>(0);
		exps 		= new Vector<Explosion>(0);
		items 		= new Vector<Item>(0);
		laser		= null;
		
		difficulte 	= 0;
		
		readOptions();
		chargerMeilleurScore();
	}
	
	public static void restoreEnvironnement()
	{
		env = new Environnement();
	}
	
	public static Environnement getEnvironnement()
	{
		return env;
	}
	
	public void setGameState( int gs)
	{
		gameState = gs;
	}
	
	public boolean isPause()
	{
		return pause != 0;
	}
	
	public boolean isPlay()
	{
		return gameState==0;
	}
	
	public boolean isEffects()
	{
		return effects == 1;
	}	
	
	public boolean goToMenu()
	{
		return pause == RETOUR_AU_MENU;
	}
	
	public void addPoints( int p )
	{
		score+=p;
	}
	
	/**
	 *  perd une vie et rinitialise les bombes
	 */
	public void perdreVie()
	{
		nombreDeVies--;
		nombreDeBombes 			 = 3;	
		tempsAvantProchaineBombe = 0;
		

		
		//pertedepoints
		score = score*90/100;
		
		if( nombreDeVies < 1 )
		{
			nombreDeVies 		= 3;
			nombreDeContinues--;
			score 				= score*90/100;
			
			items.add( new ItemTirMP( 450,-300,80 ));
		}
		else if( LVL > 1 )
			items.add( new ItemTir( 500,-200,80 ));
		
		if( nombreDeContinues < 0)
		{
			nombreDeVies 		= 0;
			nombreDeContinues 	= 0;
			gameState 			= YOU_LOOSE;
			pause 				= 40;
			
			SonManager.getSonManager().setMusiqueEnCours( SonManager.getSonManager().musiqueYouLoose );
			SonManager.getSonManager().jouerMusique();
		}
			
	}

	/**
	 * CHARGER SCORE
	 */
	public void chargerMeilleurScore()
	{
		Properties lecteur = new Properties();
		FileInputStream fichier;
		
		try
		{
			fichier = new FileInputStream("Data/scores.dat");
			lecteur.load(fichier);
			fichier.close();
			
			meilleurScore = lecteur.getProperty("score1");	
		}
		catch( Exception e)
		{
			meilleurScore = lecteur.getProperty("0000000000");
		}	
	}		
	
	/**
	 * CHARGER OPTIONS
	 */
	public void readOptions()
	{
		
		Properties lecteur = new Properties();

		FileInputStream fichier;
		
		try
		{
			fichier = new FileInputStream("Data/options.dat");
			lecteur.load(fichier);
			fichier.close();
			nombreDeContinues 	= Integer.parseInt( lecteur.getProperty("nombreDeContinus"		) ) - 1;
			antiAliasing 		= Integer.parseInt( lecteur.getProperty("antiAliasing"			) );
			effects 			= Integer.parseInt( lecteur.getProperty("effects"				) );
			difficulte			= Integer.parseInt( lecteur.getProperty("difficulte"			) );
			antiAliasingText 	= antiAliasing;
		}
		catch( Exception e)
		{
			nombreDeContinues = 2;
		}
	}
	
	/**
	 * Refresh
	 * @param t
	 */
	public void refresh(long t) 
	{
		// SI DEMANDE NOM ET SCORE
		if( pause == DEMANDER_NOM )
		{
			if( menuAskScore == null)
				menuAskScore = new MenuAskScore(score);
		
			menuAskScore.refresh(t);
			
			if ( menuAskScore.isOk() )
				pause = RETOUR_AU_MENU;
		}
		
		// SI MORT ET EN JEU
		if( pause == MODE_JEU && vivant == false)
		{
			tempsMort += t;
			
			if( tempsMort > 1500 )
			{
				vivant = true;
				invincible = 4000;
			}
		}
		
		// SI VIVANT ET EN JEU
		if( pause == MODE_JEU && vivant)
		{
			invincible -= t;
			
			if( invincible > 0)
				blink = (blink+(int)t)%200;
			else
				blink = 0;
			
			if( tempsAvantProchaineBombe > 0 )
				tempsAvantProchaineBombe -= t;
			if( tempsAvantProchaineBombe < 0 )
				tempsAvantProchaineBombe = 0;			
		}
		
		// SI MENU CHOIX
		if( pause == 2)
		{
			// dure
			boucle = System.currentTimeMillis();
			
			// affichage dans la fentre
	
			
			// Si ni haut ni bas n'est press, on remet le compteur a zero
			if( ! clavier.isHaut() && ! clavier.isBas())
				dernierDeplacement = boucle - 300;			
	
			// dplacement dans le menu
			if ( boucle - dernierDeplacement > 200 ) 
			{
				// Si haut
				if ( (clavier.isHaut() && choix == 1) || ( clavier.isBas() && choix == 0) ) 
				{
					choix = 1 - choix;
					dernierDeplacement = boucle;
					SonManager.getSonManager().jouerBeep();					
				}	
			}
			
			if( clavier.isEntrer())
			{
				clavier.entrerOk();
				
				if( choix == 0)
					pause = RETOUR_AU_JEU;
				else
					pause = DEMANDER_NOM;
			}
			
			if( clavier.isESC() )
			{
				clavier.ESCOk();
				pause = 5;
				choix = 0;				
			}
		}
	}			
	
	/**
	 * Refresh shape
	 * @param graphics
	 */
	public void shape(Graphics graphics) 
	{
		if( inGame )
		{
		
			// ON TRANSFORME LE SCORE POUR QUE CE SOIT UNE STRING DE 10 CHAR
			String scoreString = Integer.toString(score);
			for( int i=scoreString.length(); i<10;i++)
				scoreString = "0"+scoreString;
			
			graphics.drawImage( board.getImage() ,212,0, null );	
			
			
			// SCORE
			graphics.setColor( Color.white );
			graphics.setFont(new Font(Font.MONOSPACED, Font.BOLD, 15));			
			graphics.drawString(scoreString,282,17);
			
			// MEILLEUR SCORE
			graphics.setColor( Color.white );
			graphics.setFont(new Font(Font.MONOSPACED, Font.BOLD, 20));			
			graphics.drawString(meilleurScore,452,37);		
				
			// VIES
			for( int i=0; i<nombreDeVies;i++)
				graphics.drawImage( vie.getImage() ,i*20 + 217 ,721, null );
			
			// CONTINUES
			graphics.setColor( Color.white );
			graphics.setFont(new Font(Font.MONOSPACED, Font.BOLD, 12));		
			graphics.drawString(Integer.toString( nombreDeContinues )+" Coins",303,731);
			
			// BOMBES
			for( int i=0; i<nombreDeBombes;i++)
				graphics.drawImage( bombs.getImage(), i*30 + 216, 747, null );
			
			// PAUSE
			if( pause == MODE_PAUSE )
			{
				graphics.setColor( new Color( 0,0,0,80) );
				graphics.fillRect(212, 0, 600, 768);
				
				graphics.drawImage( imgPause.getImage() ,482,343, null );
			}
			else if( pause == MODE_MENU_QUITTER )
			{
				graphics.setColor( new Color( 0,0,0,80) );
				graphics.fillRect(212, 0, 600, 768);
				graphics.drawImage( retournerAuMenu[choix].getImage() ,342,268, null );			
			}
			else if( pause == DEMANDER_NOM)
			{
				if ( menuAskScore != null )
					menuAskScore.shape(graphics);
			}
		}
	}	
}
