package controles;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.util.Vector;
import javax.swing.ImageIcon;
import niveaux.*;
import elements.Joueur;
import elements.items.Item;

/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class NiveauManager extends Component
{
	private static final long serialVersionUID = 1L;
	private int 			niveau;
	private Niveau 			mapNiveau;
	private Environnement 	env;
	private Joueur 			joueur;
	
	private double 			temps;
	
	// Images
	private ImageIcon 		readyImg;
	private ImageIcon 		goImg;
	private ImageIcon 		level;
	private ImageIcon 		clear;
	private ImageIcon 		youLoose;
	
	public static NiveauManager niv; 
	
	public static NiveauManager getNiveauManager()
	{
		return niv;
	}
	
	public static void RestoreNiveauManager()
	{
		niv = new NiveauManager( Environnement.getEnvironnement().joueur );
	}
	
	private NiveauManager( Joueur j )
	{
		temps 			= 0;
		niveau 			= -1;
		env 			= Environnement.getEnvironnement();
		mapNiveau 		= new Niveau1();		
		joueur 			= j;
		env.gameState 	= Environnement.INTRO;		
		
		// Images
		readyImg 		= ImageManager.getImagemanager().getStaut(0);
		goImg 			= ImageManager.getImagemanager().getStaut(1);
		level 			= ImageManager.getImagemanager().getStaut(2);
		clear 			= ImageManager.getImagemanager().getStaut(3);
		youLoose		= ImageManager.getImagemanager().getStaut(4);
		
		niveauSuivant();
	}
	
	public Niveau getNiveau()
	{
		return mapNiveau;
	}
	
	/**
	 * Gere les niveaux
	 */
	public void niveauSuivant()
	{	
		niveau++;

		// Si le jeu n'est pas fini
		if( niveau < 4 && niveau > 0)
		{
			// On reset 
			env.items 		= new Vector<Item>(0);
			env.em			= new EnnemisManager(niveau);
			env.pause 		= 0;
			temps 			= 0;	
			env.gameState 	= Environnement.INTRO;
			env.modeBoss    = false;
			env.laser 		= null;
		
			// PLACE LE JOUEUR
			joueur.setY(770);
			joueur.setX(482);
			
			// SUPPRIME LA MUSIQUE D'INTRO POUR FAIRE DE LA PLACE //
			if( niveau == 1 ){SonManager.getSonManager().musiqueIntro = null;System.gc ();}
		
			// LANCE LA MUISQUE DU JEU
			SonManager.getSonManager().setMusiqueEnCours( SonManager.getSonManager().musiqueJeu );
			SonManager.getSonManager().jouerMusique();


		}
		else if( niveau == 0 )
		{
			// On reset 
			env.items 		= new Vector<Item>(0);
			temps 			= 0;				
			env.pause 		= 0;
			env.jeuFini 	= false;
			env.modeBoss    = false;
			env.gameState 	= Environnement.INTRO;
			env.laser 		= null;
		
			// PLACE LE JOUEUR
			joueur.setY(580);
			joueur.setX(482);
		
			// JOUE MUSIQUE
			SonManager.getSonManager().setMusiqueEnCours( SonManager.getSonManager().musiqueIntro );
		}
		else
		{
			env.pause 		= Environnement.DEMANDER_NOM;
			env.gameState 	= 0;
			env.jeuFini 	= true;
		}
		
		
		// Charge les niveaux
		if( niveau == 0 )
		{
			mapNiveau = new Niveau0();
		}
		// Charge les niveaux
		else if( niveau == 1 )
		{
			mapNiveau = new Niveau1();
		}
		else if( niveau == 2 )
		{
			// On vide la memoire
			mapNiveau = null;
			System.gc ();
			
			mapNiveau = new Niveau2();
		}
		else if( niveau == 3 )
		{
			// On vide la memoire
			mapNiveau = null;
			System.gc ();
			
			mapNiveau = new Niveau3();
		}
	}
	
	//-
	//	INTRO
	//-
	
	/**
	 * Intro refresh
	 */
	public void introRefresh()
	{
		if( temps > 200 && temps < 600 )
			SonManager.getSonManager().jouerReady();
		else if( temps > 1900 && temps < 2000 )
			SonManager.getSonManager().jouerGo();
		
		joueur.setVitesseY(-50);
	}
	
	/**
	 * Intro Shape
	 * @param graphics
	 */
	private void introShape(Graphics graphics)
	{
		if( temps < 2000)
			graphics.drawImage( readyImg.getImage(),452,340,this );
		
		else if( temps > 2000 && temps < 2200)
			graphics.drawImage( goImg.getImage(),452,340,this );			
	}
	
	
	
	
	//--
	//	LEVEL CLEAR
	//--
	
	/**
	 * Level clear Refresh
	 */
	public void levelClearRefresh()
	{
		if( env.gameState == Environnement.LEVEL_CLEAR )
		{
			if( temps > 500 && temps < 1000 )
				SonManager.getSonManager().jouerLevelClear();
			
			if( temps > 6000 )
			{
				niveauSuivant();
			}
			
			joueur.setVitesseY( -400 );
			joueur.setVitesseX(    0 );
		}
		

	}	
	
	/**
	 * Level clear shape
	 * @param graphics
	 */
	private void levelClearShape( Graphics graphics )
	{
		int alpha = (int)temps*255/6000;
		
		if( alpha > 255 )
			alpha = 255;
		
		//Fade out lors de la fin du niveau
		graphics.setColor( new Color( 1,1,1, alpha ));
		graphics.fillRect(212, 0, 600, 768);
		
		if( temps > 500 && temps < 3000)
			graphics.drawImage( level.getImage(),452,300,this );
		
		if( temps > 1000 && temps < 3000)
			graphics.drawImage( clear.getImage(),452,340,this );
	}
	
	
	//--
	//	YOU LOOSE
	//--
	
	/**
	 * Level clear Refresh
	 */
	public void youLooseRefresh()
	{
		if( temps > 500 && temps < 1000 )
			SonManager.getSonManager().jouerYouLoose();
		
		if( temps > 4000 )
		{
			env.pause 		= Environnement.DEMANDER_NOM;
			env.gameState 	= 0;
		}		
	}	
	
	/**
	 * Level clear shape
	 * @param graphics
	 */
	private void youLooseShape( Graphics graphics )
	{	
		int alpha = ((int)temps*255/4000);
		if( alpha > 255 )
			alpha = 255;
		
		
		//Fade out lors de la fin du niveau
		graphics.setColor( new Color( 1,1,1, alpha ) );
		graphics.fillRect(212, 0, 600, 768);
		
		if( temps > 500 && temps < 3000)
			graphics.drawImage( youLoose.getImage(),452,300,this );

	}
		
	
	
	/**
	 * Refresh Global
	 * @param t
	 */
	public void refresh(long t) 
	{
		if( niveau > 0 )
		{
			if( env.gameState == Environnement.INTRO || env.gameState == Environnement.LEVEL_CLEAR || env.gameState == Environnement.YOU_LOOSE)
				temps+=t;
	
			if( env.gameState == Environnement.MODE_JEU)
				temps = 0;
			
			if( env.gameState == Environnement.INTRO )
			{
				if( temps < 2200)
				{
					introRefresh();
				}
				else
					if( ! env.isPlay())
						env.gameState=0;
			}
			else if(  env.gameState == Environnement.LEVEL_CLEAR )
			{
				levelClearRefresh();
			}
			else if(  env.gameState == Environnement.YOU_LOOSE )
			{
				if( temps < 5300)
				{
					youLooseRefresh();
				}
				else
					if( ! env.isPlay())
						env.gameState=0;
			}		
			
			if( ! env.isPause() )	
				mapNiveau.refresh(t);		
		}
		else
		{
			mapNiveau.refresh(t);
			
			if( ((Niveau0)mapNiveau).finIntro() )
				niveauSuivant();
		}
	}		
	
	
	
	
	
	/**
	 * Shape global background
	 * @param graphics
	 */
	public void shapeBackground(Graphics graphics) 
	{
		mapNiveau.shapeBackground(graphics);	
	}
	
	
	/**
	 * Shape global foreground
	 * @param graphics
	 */
	public void shapeForeground(Graphics graphics) 
	{
		mapNiveau.shapeForeground(graphics);
		
		if( niveau > 0 )
		{
			if(temps < 2200 && env.gameState == Environnement.INTRO )
				introShape(graphics);	
			
			else if(  env.gameState == Environnement.LEVEL_CLEAR )
				levelClearShape( graphics );
			
			else if(  env.gameState == Environnement.YOU_LOOSE )
				youLooseShape( graphics );	
		}
	}		
}
