package elements.Ennemis.Boss;

import java.awt.Graphics;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.util.Random;
import java.util.Vector;

import controles.ImageManager;
import controles.SonManager;
import elements.Tirs.Bombe;
import elements.Tirs.Laser;
import elements.Tirs.Tir;
import elements.Tirs.TirEnnemi;
import elements.Tirs.TirEnnemiInvincible;
import elements.items.ItemBombe;
import elements.items.ItemPoint;
import elements.items.ItemTir;
import elements.Ennemis.Ennemi;
import elements.Explosions.Explosion;
import elements.Explosions.ExplosionEnnemi;

/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class Boss1 extends Boss
{
	private Boss1Sub[] elements;
	private double rotation;
	
	public Boss1()
	{
		super( 512 - ImageManager.getImagemanager().getBoss(0).getIconWidth()/2 ,-300,0,100,15000,6000,100,0,400,2000,4);
		
		img = imgLoad.getBoss(0);
		
		elements = new Boss1Sub[2];
		
		elements[0] = new Boss1Sub();
		elements[1] = new Boss1Sub();
	}
	
	/**
	 * Recupre la forme de l'ennemi
	 */	
	public Area getArea()
	{
		Area a1 = new Area( new Ellipse2D.Double(x+140,y+15,125,125) );
		a1.add( new Area( new Rectangle2D.Double(x,y+68,400,15) ) );
		
		if( elements[0] != null )	
			a1.add( elements[0].getArea() );
		
		if( elements[1] != null )	
			a1.add( elements[1].getArea() );
		
		return a1;
	}	
	
	
	//--------
	//
	// TIRS
	//
	//--------
	
	
	/**
	 * Recupre la position X du tir
	 * @return position X du tir
	 */	
	public int getPointTirX( int angle, int pos )
	{
		return (int) (Math.cos( Math.toRadians(rotation-angle) )*pos + x + getWidth()/2);
	}
	
	/**
	 * Recupre la position Y du tir
	 * @return position Y du tir
	 */
	public int getPointTirY( int angle, int pos)
	{
		return (int)(Math.sin( Math.toRadians(rotation-angle) )*pos + y + getHeight()/2);
	}	
	
	/**
	 * Recupre la vitesse en X du tir
	 * @return vitesse en X du tir
	 */	
	public int getPointTirVitesseX( int angle,int vitesse )
	{
		return (int) (Math.cos( Math.toRadians(rotation-angle) )*vitesse);
	}
	
	/**
	 * Recupre la vitesse en Y du tir
	 * @return vitesse en Y du tir
	 */
	public int getPointTirVitesseY( int angle, int vitesse )
	{
		return (int)(Math.sin( Math.toRadians(rotation-angle) )*vitesse);
	}	
	
	
	
	/**
	 * 
	 * GERE LA ROUTINE DES TIRS
	 * 
	 */
	public void gestionTirs()
	{
		if( pretTirer() )
		{
			for(int i=0;i<360;i+=45)
			{
				for(int j=100;j<400;j+=100)
					tirs.add( new TirEnnemi( getPointTirX(i,0), getPointTirY(i,0) , getPointTirVitesseX(i,j), getPointTirVitesseY(i,j)) );
			
				tirs.add( new TirEnnemi( getPointTirX(i,50), getPointTirY(i,50) , getPointTirVitesseX(i,200), getPointTirVitesseY(i,200)) );
				tirs.add( new TirEnnemiInvincible( getPointTirX(i,150), getPointTirY(i,150) , getPointTirVitesseX(i,200), getPointTirVitesseY(i,200)) );
			}
			SonManager.getSonManager().jouerExpBoss();
			tirOk();
		}
	}		
	
	
	//---------
	//
	//	OBJETS
	//
	//---------
	
	/**
	 * Ajoute les points dans un vecteur
	 */
	protected void addPoints()
	{
		Random r = new Random();
		
		// Autres points
		for( int i=0;i<30; i++ )
		{
			it.add
			( 
					new ItemPoint
					( 
							getX() + (int)(r.nextFloat()*getWidth()), 
							getY() + (int)(r.nextFloat()*getHeight()),
							(int)(80 + 80 * r.nextFloat()),
							getPoints()/5
					) 
			);	
		}
		
		it.add( new   ItemTir( getX() + getWidth()/2 + 50, getY() + getHeight()/2 - 20,(int)(100)) );		
		it.add( new   ItemTir( getX() + getWidth()/2 - 50, getY() + getHeight()/2 - 20,(int)(100)) );			
		it.add( new ItemBombe( getX() + getWidth()/2     , getY() + getHeight()/2 + 20,(int)(100)) );	
		
	}	
	
	protected void addExplosions()
	{
		Random r = new Random();
		
		// Autres points
		for( int i=0;i<60; i++ )
		{
			exp.add
			( 
					new ExplosionEnnemi
					( 
							getX() + (int)(r.nextFloat()*getWidth()), 
							getY() + (int)(r.nextFloat()*getHeight())
					) 
			);	
		}
		
		SonManager.getSonManager().jouerExplosionBoss();
	}	
	
	/**
	 * Mort du boss
	 */
	protected void die()
	{
		super.die();
		addExplosions();
		addPoints();
	}
	
	
	/**
	 * Recupere les tirs
	 */
	public Vector<TirEnnemi> getTirs()
	{
		if( elements[0] != null )
		{
			tirs.addAll( elements[0].getTirs() );
			elements[0].clearTirs();
		}
		
		if( elements[1] != null )	
		{
			tirs.addAll( elements[1].getTirs() );
			elements[1].clearTirs();
		}
		
		return tirs;
	}	
	
	
	
	/**
	 * Recupere les Explosions
	 */
	public Vector<Explosion> getExplosions()
	{
		if( elements[0] != null )
		{
			exp.addAll( elements[0].getExplosions() );
			elements[0].clearExplosions();
		}
		
		if( elements[1] != null )	
		{
			exp.addAll( elements[1].getExplosions() );
			elements[1].clearExplosions();
		}
		
		return exp;
	}		

	
	/**
	 * Collsion entre un ennemi et un tir
	 * @param t
	 */
	public void collision( Tir t)
	{
		Area t1 = new Area( new Rectangle2D.Double( t.getX() , t.getY(), t.getWidth(), t.getHeight() ));
		Area a1 = new Area( new Ellipse2D.Double(x+140,y+15,125,125) );
		
		t1.intersect( a1 );
		
		if( ! t1.isEmpty() )
		{
			if( y>0 )
			{
				ennergie-= t.getForce();
				hit 	 = true;
			}
			
			t.explose();
		}
		
		if( elements[0] != null )		
			elements[0].collision(t);
		if( elements[1] != null )		
			elements[1].collision(t);
		
		// SI mort on s'occupe des points et des explosions
		if( ennergie <=0 && it.size() == 0 )
			die();
	}	
	
	/**
	 * Collision ennemi / bombe
	 * @param i item
	 */	
	public void collision( Bombe b )
	{
		super.collision(b);

		if( elements[0] != null )		
			elements[0].collision(b);
		if( elements[1] != null )		
			elements[1].collision(b);	
		
		// SI mort on s'occupe des points et des explosions
		if( ennergie <=0 && it.size() == 0 )
			die();
	}		
	
	/**
	 * Collision ennemi / laser
	 * @param i item
	 */	
	public boolean collision( Laser l )
	{
		boolean ok = false;
		
		Area a1 = new Area( new Ellipse2D.Double(x+140,y+15,125,125) );
		Area a2 = l.getArea();
		a2.intersect(a1);

		//Si il y a collision
		if(  ! a2.isEmpty() && y>0 )
		{
			l.setMaxY( (int)a2.getBounds2D().getCenterY() );
			ok 		 = true;
			
			// Si n'est plus immunis
			if( tempsImuniseBombe <= 0 )
			{
				ennergie		   -= l.getForce();
				hit		 			= true;
				tempsImuniseBombe 	= l.getTempsImunise();	
			}
		}

		if( elements[0] != null && elements[0].collision(l))		
			ok = true;
		if( elements[1] != null && elements[1].collision(l))	
			ok = true;	
		
		// SI mort on s'occupe des points et des explosions
		if( ennergie <=0 && it.size() == 0 )
			die();
		
		return ok;
	}			
	
	public void refresh(long temps) 
	{
		if( y > 10)
		{
			vitesseX = 0;
			vitesseY = 0;
		}
		
		super.refresh(temps);
		
		rotation += (200*temps)/1000.0;
		
		if( rotation >= 360)
			rotation -= 360;
		
		if( rotation < 0)
			rotation += 360;		

		if( elements[0] != null )
		{
			elements[0].refresh(temps);
			elements[0].setPos( (int) x-10, (int)y+41);
			exp.addAll( elements[0].getExplosions() );
			elements[0].clearExplosions();
		}
		if( elements[1] != null )		
		{
			elements[1].refresh(temps);			
			elements[1].setPos( (int)x + img.getIconWidth() + 10 - 64, (int)y+41);
			exp.addAll( elements[1].getExplosions() );
			elements[1].clearExplosions();
		}
		
		if( elements[0] != null && elements[0].isDead() )
			elements[0] = null;
		
		if( elements[1] != null && elements[1].isDead() )
			elements[1] = null;
		
	}	
	
	public void shape(Graphics graphics) 
	{
		super.shape(graphics);

		if( elements[0] != null )		
			elements[0].shape(graphics);
		if( elements[1] != null )		
			elements[1].shape(graphics);
	}	
}

////////////////////////////////
////////////////////////////////
///////////////////////////////

//	SOUS BOSS

////////////////////////////////
////////////////////////////////
///////////////////////////////

class Boss1Sub extends Ennemi
{
	private int rotation;
	
	public Boss1Sub()
	{
		super( 0,0,0,0,3000,700,0,0,300,4000,2);
		img = imgLoad.getBoss(1);
	}
	
	public void setPos( int xx, int yy)
	{
		x = xx;
		y = yy;
	}
	
	/**
	 * Recupre la forme de l'ennemi
	 */	
	public Area getArea()
	{
		return new Area( new Ellipse2D.Double(x+5,y+5,53,53) );
	}	
	
	
	//-------
	//
	//	TIRS
	//
	//-------
	
	/**
	 * Recupre la position X du tir
	 * @return position X du tir
	 */	
	public int getPointTirX( int angle, int pos )
	{
		return (int) (Math.cos( Math.toRadians(rotation-angle) )*pos + x + getWidth()/2);
	}
	
	
	/**
	 * Recupre la position Y du tir
	 * @return position Y du tir
	 */
	public int getPointTirY( int angle, int pos)
	{
		return (int)(Math.sin( Math.toRadians(rotation-angle) )*pos + y + getHeight()/2);
	}	

	
	/**
	 * Recupre la vitesse en X du tir
	 * @return vitesse en X du tir
	 */	
	public int getPointTirVitesseX( int angle,int vitesse )
	{
		return (int) (Math.cos( Math.toRadians(rotation-angle) )*vitesse);
	}
	
	
	/**
	 * Recupre la vitesse en Y du tir
	 * @return vitesse en Y du tir
	 */
	public int getPointTirVitesseY( int angle, int vitesse )
	{
		return (int)(Math.sin( Math.toRadians(rotation-angle) )*vitesse);
	}
	
	
	/**
	 * Gere la routine des tirs
	 */
	public void gestionTirs()
	{
		if( pretTirer() )
		{

			for(int i=0;i<360;i+=90)
			{
				for(int j=0;j<300;j+=100)
					tirs.add( new TirEnnemi( getPointTirX(i,j), getPointTirY(i,j) , getPointTirVitesseX(i,150), getPointTirVitesseY(i,150)) );
			
				tirs.add( new TirEnnemiInvincible( getPointTirX(i,50), getPointTirY(i,50) , getPointTirVitesseX(i,200), getPointTirVitesseY(i,200)) );
			}
			SonManager.getSonManager().jouerExpBoss();
			tirOk();
		}
	}		
	
	
	/**
	 * Collsion entre un ennemi et un tir
	 * @param t
	 */
	public void collision( Tir t)
	{
		Area t1 = new Area( new Rectangle2D.Double( t.getX() , t.getY(), t.getWidth(), t.getYPrededent() ));
		
		t1.intersect( getArea() );
		
		if( ! t1.isEmpty() )
		{
			if( y>0 )
			{
				ennergie-= t.getForce();
				hit		 = true;
			}
			t.explose();
		}
	}
	
	
	/**
	 * Refresh
	 */
	public void refresh(long temps) 
	{
		super.refresh(temps);
		
		rotation += (200*temps)/1000.0;
		
		if( rotation >= 360)
			rotation -= 360;
		
		if( rotation < 0)
			rotation += 360;
	}		
}
