package elements.Ennemis;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Vector;

import controles.Environnement;
import controles.ImageManager;
import controles.SonManager;
import elements.Dessinable;
import elements.Explosions.*;
import elements.Tirs.*;
import elements.items.Item;
import elements.items.ItemBombe;
import elements.items.ItemPoint;
import elements.items.ItemTir;

/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class Ennemi extends Dessinable implements Comparable<Ennemi>
{
	private static final long serialVersionUID = 1L;
	
	public static final int RIEN = 0;
	public static final int ITEM_TIR = 1;
	public static final int ITEM_BOMBE = 2;
	public static final int ITEM_TIR_ET_BOMBE = 3;
	
	protected boolean 		aTueLeJoueur;
	protected int 			evol;
	protected float 		vitesseX;
	protected float 		vitesseY;
	protected int 			ennergie;
	protected int 			points;
	protected int 			tempsImuniseBombe;
	protected ImageManager 	imgLoad;
	protected boolean		hit;
	
	// GESTION TIRS
	
	protected int delaisEntreDeuxCoups;
	protected int delaisEntreDeuxSalves;
	protected int delaisCoup;	
	protected int delaisSalve;	
	protected int nombreCoupParSalve;		
	protected int nombreCoup;
	protected int bonus;
	

	protected Vector<TirEnnemi> tirs;
	protected Vector<Explosion> exp;
	protected Vector<Item> it;

	/**
	 * Constructeur
	 * @param xx x
	 * @param yy y
	 * @param vx vitesse en X
	 * @param vy vitesse en Y
	 * @param e energie
	 * @param p point recuperes en le tuant
	 * @param ev numer d'evolution
	 */
	protected Ennemi(int xx, int yy, int vx, int vy, int e, int p, int ev, int b)
	{
		super(xx,yy);
		
		evol = ev;
		vitesseX = vx;
		vitesseY = vy;
		ennergie = e;
		points = p;
		tempsImuniseBombe = 0;
		bonus = b;
		imgLoad = ImageManager.getImagemanager();
		
		// Tirs
		delaisEntreDeuxCoups = 50;
		delaisEntreDeuxSalves = 500;
		nombreCoupParSalve = 0;
		delaisSalve = 0;		
		delaisCoup = 0;
		nombreCoup = 0;
		
		init();
	}	
	
	/**
	 * Constructeur
	 * @param xx
	 * @param yy
	 * @param vx
	 * @param vy
	 * @param e
	 * @param p
	 * @param ev
	 * @param b
	 * @param dc
	 * @param ds
	 * @param ncs
	 */
	protected Ennemi(int xx, int yy, int vx, int vy, int e, int p, int ev,int b, int dc, int ds, int ncs)
	{
		super(xx,yy);
		
		evol = ev;
		vitesseX = vx;
		vitesseY = vy;
		ennergie = e;
		points = p;
		tempsImuniseBombe = 0;
		bonus = b;
		imgLoad = ImageManager.getImagemanager();	
		
		// Tirs
		delaisEntreDeuxCoups = dc;
		delaisEntreDeuxSalves = ds;
		nombreCoupParSalve = ncs;
		delaisSalve = 0;		
		delaisCoup = 0;
		nombreCoup = 0;
		
		init();
	}
	
	/**
	 * init
	 */
	public void init()
	{
		tirs = new Vector<TirEnnemi>(0);	
		exp	 = new Vector<Explosion>(0);
		it	 = new Vector<Item>(0); 
	}
	
	
	
	
	/**
	 * Recupre la vitesse en X
	 * @return vitesse en X
	 */
	public int getVitesseX()
	{
		return (int)vitesseX;
	}
	
	
	/**
	 * Recupre la vitesse en Y
	 * @return vitesse en Y
	 */
	public int getVitesseY()
	{
		return (int)vitesseY;
	}
	
	
	/**
	 * Recupre les points
	 * @return points
	 */
	public int getPoints()
	{
		return points;
	}
	
	
	/**
	 * Largeur
	 */
	public int getWidth()
	{
		return img.getImage().getWidth(null);
	}
	
	
	/**
	 * Hauteur
	 */
	public int getHeight()
	{
		return img.getImage().getHeight(null);
	}	
	
	
	/**
	 * Recupre le numero d'evolution
	 * @return numero d'evolution
	 */
	public int getEvol()
	{
		return evol;
	}
	
	/**
	 * Drop t-on un tir
	 * @return drop tir
	 */
	public boolean getItemTir()
	{
		return ( bonus == ITEM_TIR || bonus == ITEM_TIR_ET_BOMBE );
	}
	
	/**
	 * Drop t-on une bombe
	 * @return drope une bombe
	 */
	public boolean getItemBombe()
	{
		return ( bonus == ITEM_BOMBE || bonus == ITEM_TIR_ET_BOMBE );
	}
	
	
	/**
	 * Ennergie de l'ennemie
	 * @return ennergie
	 */
	public int getEnnergie()
	{
		return ennergie;
	}
	
	/**
	 * vrai si l'ennemi est mort
	 * @return vrai si l'ennemi est mort
	 */
	public boolean isDead()
	{
		return ennergie <= 0;
	}	
	
	/**
	 * vrai si l'ennemi est mort
	 * @return vrai si l'ennemi est mort
	 */
	public boolean isKilled()
	{
		return ennergie <= 0;
	}			
	
	
	/**
	 * place l'ennemi en X
	 * @param xx position X
	 */
	public void setX( int xx )
	{
		x = xx;
	}

	
	/**
	 * place l'ennemi en Y
	 * @param yy position y
	 */	
	public void setY( int yy )
	{
		y = yy;
	}
	
	
	/**
	 * Choisi vitesse ennemi en X
	 * @param vx vitesse ennemi en X
	 */
	public void setVitesseX( int vx )
	{
		vitesseX = vx;
	}
	
	
	/**
	 * Choisi vitesse ennemi en Y
	 * @param vy vitesse ennemi en Y
	 */
	public void setVitesseY( int vy )
	{
		vitesseY = vy;
	}
	
	
	/**
	 * Vrai si l'ennemi a tu le joueur
	 * @return l'ennemi a tu le joueur
	 */
	public boolean getATueLeJoueur()
	{
		return aTueLeJoueur;
	}
	
	
	/**
	 * Choisi si l'ennemi a tue le joueur ( a mettre a faux aprs traitement )
	 * @param b l'ennemi a tu le joueur
	 */
	public void setATueLeJoueur( boolean b)
	{
		aTueLeJoueur = b;
	}
	

	/**
	 * Recupre la forme de l'ennemi
	 * @return forme de l'ennemi
	 */	
	public Area getArea()
	{
		return new Area( new Rectangle2D.Double(x,y,getWidth(),getHeight()) );
	}
	
	/**
	 * Routine de mort de l'ennemi
	 */
	protected void die()
	{
		addPoints();
	}
	
	/**
	 * Tue l'ennemi
	 */
	public void kill()
	{
		ennergie = -1;
	}
	
	/**
	 * Drop les points
	 */
	protected void addPoints()
	{
		it.add
		( 
				new ItemPoint
				( 
						getX() + getWidth()/2 - 15, 
						getY() + getHeight()/2,
						getVitesseY(),
						getPoints() * 10
				) 
		);
		
		if( bonus == ITEM_TIR || bonus == ITEM_TIR_ET_BOMBE )
				it.add
				( 
						new ItemTir
						( 
								getX() + getWidth()/2 - 15, 
								getY() + getHeight()/2 - 20,
								(int)(getVitesseY()*0.7)
						) 
				);	
		
		if( bonus == ITEM_BOMBE || bonus == ITEM_TIR_ET_BOMBE )
			it.add
			( 
					new ItemBombe
					( 
							getX() + getWidth()/2 - 15, 
							getY() + getHeight()/2 + 20,
							(int)(getVitesseY()*0.7)
					) 
			);	
			

	}
	
	
	/**
	 * Collsion entre un ennemi et un tir
	 * @param t
	 */
	public void collision( Tir t)
	{
		if
		( 	
				t.getX() + t.getWidth()*2  	>= x && 
				t.getX() - t.getWidth()		<= x+img.getImage().getWidth(null) && 
				( 
						( t.getY() < y && t.getYPrededent() > y+img.getImage().getHeight(null) ) || 
						( t.getY() > y && t.getY() < y+img.getImage().getHeight(null) )
				) 
		)
		{
			if( y>0 )
			{
				ennergie-= t.getForce();
				hit = true;
			}
			t.explose();
		}
		
		// SI mort on s'occupe des points
		if( ennergie <=0 && it.size() == 0 )
		{
			die();
		}
	}	
	

	/**
	 * Collision ennemi / player
	 * @param i item
	 */	
	public void collision( Bombe b )
	{
		Area a1 = getArea();
			
		Area t1 = b.getArea();
		Area t2 = b.getAreaInterne();
		
		t1.intersect(a1);
		t2.intersect(a1);

		if(  ! t1.isEmpty() && t2.isEmpty() && tempsImuniseBombe == 0 && y>0)
		{
			SonManager.getSonManager().jouerExplosion();
			exp.add( new ExplosionEnnemi( (int)t1.getBounds().getCenterX(), (int) t1.getBounds().getMinY() - 100 ) );
			ennergie-= b.getForce();
			hit		 = true;
			tempsImuniseBombe = 600;
		}
		
		// SI mort on s'occupe des points
		if( ennergie <=0 && it.size() == 0 )
			die();
	}	
	
	
	/**
	 * Collsion entre un ennemi et un tir
	 * @param t
	 */
	public boolean collision( Laser l)
	{
		Area a2 = l.getArea();
		a2.intersect(getArea());

		//Si il y a collision
		if(  ! a2.isEmpty() && y>0 )
		{
			l.setMaxY( (int)a2.getBounds2D().getCenterY() );
			
			if( tempsImuniseBombe <= 0 )
			{
				ennergie		   -= l.getForce();
				tempsImuniseBombe 	= l.getTempsImunise();	
				hit		 			= true;
			}
		}
		else
			return false;
		
		// SI mort on s'occupe des points
		if( ennergie <=0 && it.size() == 0 )
			die();
		
		return true;
	}		
	
	/**
	 * Recupere les items
	 * @return items
	 */
	public Vector<Item> getItems()
	{
		return it;
	}
	
	/**
	 * Recupere les tirs
	 * @return tirs
	 */
	public Vector<TirEnnemi> getTirs()
	{
		return tirs;
	}
	
	/**
	 * Recupere les explosions
	 * @return explosions
	 */
	public Vector<Explosion> getExplosions()
	{
		return exp;
	}	
	
	/**
	 * Vide les objets
	 */
	public void clearItems()
	{
		if( it.size() > 0 )
			it = new Vector<Item>(0);
	}
	
	/**
	 * Vide les tirs
	 */
	public void clearTirs()
	{
		if( tirs.size() > 0 )
			tirs = new Vector<TirEnnemi>(0);
	}
	
	/**
	 * Vide les explosions
	 */
	public void clearExplosions()
	{
		if( exp.size() > 0 )
			exp = new Vector<Explosion>(0);
	}	
	
	/**
	 * Gere la mecanique des tries
	 */
	public void gestionTirs()
	{
		if( pretTirer() )
		{
			tirs.add( new TirEnnemi( (int)x+img.getIconWidth()/2, (int)y+img.getIconHeight()/2, (int)(vitesseX*1.5), (int)(vitesseY*1.5)) );
			tirOk();
		}
	}
	
	/**
	 * L'ennemi est il pret a tirer ?
	 * @return bolleen
	 */
	public boolean pretTirer()
	{
		return nombreCoup < nombreCoupParSalve && delaisCoup >= delaisEntreDeuxCoups;
	}
	
	
	/**
	 * L'ennemi a tir
	 */
	public void tirOk()
	{
		if( nombreCoupParSalve > 0)
		{
			nombreCoup++;
			delaisCoup = 0;
		}
	}
	
	/**
	 * L'objet peut-il entrer en collision avec le joueur
	 * @return collision possible
	 */
	public boolean isPhysique()
	{
		return true;
	}
			
	/**
	 * Raffraichi les tirs
	 * @param temps temps
	 */
	public void refreshTirs( long temps )
	{
		// GESTION DES COUPS
		if( nombreCoup < nombreCoupParSalve )
		{
			delaisCoup += temps;
		}
		else
		{
			if( delaisSalve >= delaisEntreDeuxSalves )
			{
				delaisSalve = 0;
				nombreCoup = 0;
				delaisCoup = 0;
			}
			else
			{
				delaisSalve += temps;
			}
		}			
	
		gestionTirs();
	}
	
	
	/**
	 * rafraichi positions
	 */
	public void refresh(long temps) 
	{
		// refresh en fonction de la vtesse
		x += (vitesseX*temps)/1000.0;
		y += (vitesseY*temps)/1000.0;	
		
		if( tempsImuniseBombe > 0)
			tempsImuniseBombe -= temps;
		
		if( tempsImuniseBombe < 0 )
			tempsImuniseBombe = 0;

		// Si l'ennemi est a l'ecran
		if( y > 0)
		{
			refreshTirs( temps );
		}
	
		// Si mort ennemi
		if( ennergie <= 0 )
		{
			// Joue le son
			SonManager.getSonManager().jouerExplosionBig();
			// Creer une explosion
			exp.add( new ExplosionEnnemi(getX() + getWidth()/2 ,getY() - getHeight()/2) );
		}
	}	
	
	
	/**
	 * Redessine
	 */
	public void shape(Graphics graphics) 
	{
		// Affiche image
		graphics.drawImage( img.getImage(), (int)x, (int)y, null );
		
		// Flash blanc si touch
		if( hit && Environnement.getEnvironnement().effects == 1)
		{
			BufferedImage imageBlanche= new BufferedImage( img.getIconWidth(), img.getIconHeight(), BufferedImage.TYPE_INT_ARGB);
			Graphics2D g = imageBlanche.createGraphics();
			
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
			g.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION,RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
			
			g.drawImage(img.getImage(), 0, 0, null);
			
			for( int i=0; i<img.getIconWidth();i++)
				for( int j=0; j<img.getIconHeight();j++)
				{
					int rgb = imageBlanche.getRGB(i,j);
					int alpha = (rgb >>24 ) & 0xFF;

					if( alpha > 0 )
						imageBlanche.setRGB(i, j, ( (alpha/5)<<24)+(255<<16)+(255<<8)+255 );
				}
			
			graphics.drawImage( imageBlanche, (int)x, (int)y,null);	
			hit = false;
		}	
	}
	
	public void shapeForeground(Graphics graphics) {}		

	
	/**
	 * Permet de trier les ennemis
	 */
	public int compareTo(Ennemi e) 
	{
		return - new Integer( (int) y ).compareTo( new Integer( (int) e.y ));
	}
	
	public String getType()
	{
		return "Ennemi";
	}
	
}
