package elements.Ennemis;

import java.awt.Graphics;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;
import elements.Tirs.TirEnnemi;


/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class EnnemiEvol2 extends Ennemi
{
	private static final long serialVersionUID = 1L;
	
	private int stopLimit;
	private int rotation;
	private int vitesseR;
	
	private ImageIcon img2;

	
	/**
	 * Constructeur
	 * @param xx x
	 * @param yy y
	 * @param vx vitesseX
	 * @param vy vitesseY
	 * @param strop position d'arret du vaisseau
	 * @param il imageLoader
	 */
	public EnnemiEvol2(int xx,int yy, int vx, int vy, int b, int stop)
	{
		super(xx,yy,vx,vy,600,100,2,b,100,3000,20);
		
		stopLimit = stop;
		vitesseR = 250;
		
		img  = imgLoad.getEnnemi(3);
		img2 = imgLoad.getEnnemi(2);		
	}

	
	/**
	 * Recupre la position X du tir
	 * @return position X du tir
	 */	
	public int getPointTirX()
	{
		return (int) (Math.cos( Math.toRadians(rotation-90) )*30 + x + 24);
	}
	
	
	/**
	 * Recupre la position Y du tir
	 * @return position Y du tir
	 */
	public int getPointTirY()
	{
		return (int)(Math.sin( Math.toRadians(rotation-90) )*30 + y + 24);
	}	

	
	/**
	 * Recupre la vitesse en X du tir
	 * @return vitesse en X du tir
	 */	
	public int getPointTirVitesseX()
	{
		return (int) (Math.cos( Math.toRadians(rotation-90) )*150);
	}
	
	
	/**
	 * Recupre la vitesse en Y du tir
	 * @return vitesse en Y du tir
	 */
	public int getPointTirVitesseY()
	{
		return (int)(Math.sin( Math.toRadians(rotation-90) )*150);
	}
	
	
	public void gestionTirs()
	{
		if( pretTirer() )
		{
			tirs.add( new TirEnnemi( getPointTirX(), getPointTirY() , getPointTirVitesseX(), getPointTirVitesseY() ) );
			tirOk();
		}
	}	

	
	/**
	 * Recupre la forme de l'ennemi
	 */	
	public Area getArea()
	{
		return new Area( new Ellipse2D.Double(x,y,getWidth(),getHeight()) );
	}	
	
	
	
	/**
	 * Rafraichi les coordonnes
	 */
	public void refresh(long temps) 
	{
		if( y > stopLimit)
		{
			vitesseX = 0;
			vitesseY = 0;
		}
		
		super.refresh(temps);
		
		rotation += (vitesseR*temps)/1000.0;
		
		if( rotation >= 360)
			rotation -= 360;
		
		if( rotation < 0)
			rotation += 360;
		
	}	
	
	
	/**
	 * Dessine
	 */
	public void shape(Graphics graphics) 
	{
		super.shape(graphics);
		
		//Cree un buffer vide
		BufferedImage imageRot= new BufferedImage( img2.getImage().getHeight(null), img2.getImage().getWidth(null), BufferedImage.TYPE_INT_ARGB);

		// Copie l'image dans un buffer
        BufferedImage bufferedImage = new BufferedImage(img2.getImage().getWidth(null), img2.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);
        Graphics g = bufferedImage.createGraphics();
        g.drawImage(img2.getImage(), 0, 0, null);
        g.dispose();
		 
        // On tourne l'image dans le buffer
		AffineTransform tx = new AffineTransform();
		tx.rotate((Math.toRadians(rotation)),( img2.getImage().getWidth(null) /2),( img2.getImage().getHeight(null) )/2);
		AffineTransformOp op = new AffineTransformOp(tx,  AffineTransformOp.TYPE_BILINEAR);
	
		// On met l'image tourne dans ImageRot
		op.filter(bufferedImage, imageRot);
		 
		// On affiche
		graphics.drawImage( imageRot, (int)x, (int)y,null);
	}	
	
}