package elements.Ennemis;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;

import controles.SonManager;

import elements.Explosions.ExplosionEnnemi;
import elements.Tirs.TirEnnemi;
import elements.Tirs.Tir;


/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class EnnemiEvol3 extends Ennemi
{
	private static final long serialVersionUID = 1L;
	
	private int stopLimit;
	private int rotation;

	private ImageIcon img2;

	private int timer;
	
	/**
	 * Constructeur
	 * @param xx x
	 * @param yy y
	 * @param vx vitesseX
	 * @param vy vitesseY
	 * @param strop position d'arret du vaisseau
	 * @param il imageLoader
	 */
	public EnnemiEvol3(int xx,int yy, int vx, int vy, int b, int stop)
	{
		super(xx,yy,vx,vy,800,30,3,b,0,1000,1);
		
		stopLimit = stop;
		
		img  = imgLoad.getEnnemi(5);
		img2 = imgLoad.getEnnemi(4);	
		timer = 5000;
	}

	
	/**
	 * Recupre la position X du tir
	 * @return position X du tir
	 */	
	public int getPointTirX( int angle, int position )
	{
		return (int) (Math.cos( Math.toRadians(rotation-angle) )*position + x + 24);
	}
	
	
	/**
	 * Recupre la position Y du tir
	 * @return position Y du tir
	 */
	public int getPointTirY( int angle, int position )
	{
		return (int)(Math.sin( Math.toRadians(rotation-angle) )*position + y + 24);
	}	

	
	/**
	 * Recupre la vitesse en X du tir
	 * @return vitesse en X du tir
	 */	
	public int getPointTirVitesseX( int angle , int vitesse)
	{
		return (int) (Math.cos( Math.toRadians(rotation-angle) )*vitesse);
	}
	
	
	/**
	 * Recupre la vitesse en Y du tir
	 * @return vitesse en Y du tir
	 */
	public int getPointTirVitesseY( int angle, int vitesse)
	{
		return (int)(Math.sin( Math.toRadians(rotation-angle) )*vitesse);
	}
	
	
	public void gestionTirs()
	{
		if( pretTirer() )
		{
			tirs.add( new TirEnnemi( getPointTirX(  0,30), getPointTirY(  0,30) , getPointTirVitesseX(  0,150), getPointTirVitesseY(  0,150) ) );
			tirs.add( new TirEnnemi( getPointTirX( 90,30), getPointTirY( 90,30) , getPointTirVitesseX( 90,150), getPointTirVitesseY( 90,150) ) );
			tirs.add( new TirEnnemi( getPointTirX(180,30), getPointTirY(180,30) , getPointTirVitesseX(180,150), getPointTirVitesseY(190,150) ) );
			tirs.add( new TirEnnemi( getPointTirX(270,30), getPointTirY(270,30) , getPointTirVitesseX(270,150), getPointTirVitesseY(270,150) ) );
			tirOk();
		}
	}	

	
	/**
	 * Recupre la forme de l'ennemi
	 */	
	public Area getArea()
	{
		return new Area( new Ellipse2D.Double(x+15,y+15,getWidth()-15,getHeight()-15) );
	}	
	
	
	public void explose()
	{
		for( int i=0;i<360;i+=30)
			tirs.add( new TirEnnemi( getPointTirX(i,2), getPointTirY(i,2) , getPointTirVitesseX(i,300), getPointTirVitesseY(i,300) ) );
		
		ennergie = 0;
	}
	
	/**
	 * vrai si l'ennemi est mort
	 * @return vrai si l'ennemi est mort
	 */
	public boolean isKilled()
	{
		return timer >= 0 && ennergie<=0;
	}	
	
	
	/**
	 * Collsion entre un ennemi et un tir
	 * @param t
	 */
	public void collision( Tir t)
	{
		if
		( 	
				t.getX() + t.getWidth()*2  	>= x+15 && 
				t.getX() - t.getWidth()		<= x+img.getImage().getWidth(null)-15 && 
				( 
						( t.getY() < y+15 && t.getYPrededent() > y+img.getImage().getHeight(null)-10 ) || 
						( t.getY() > y+15 && t.getY() < y+img.getImage().getHeight(null)-10 )
				) 
		)
			
			
		{
			if( y>0 )
				ennergie-= t.getForce();
			t.explose();
		}
		
		// SI mort on s'occupe des points
		if( ennergie <=0 && it.size() == 0 )
			die();
	}		
	
	
	/**
	 * Rafraichi les coordonnes
	 */
	public void refresh(long temps) 
	{
		if( y > stopLimit)
		{
			vitesseX = 0;
			vitesseY = 0;
		}
		
		super.refresh(temps);
		
		rotation += (temps)/10.0;
		
		if( rotation >= 360)
			rotation -= 360;
		
		if( rotation < 0)
			rotation += 360;
		
		if( y > 10 )
			timer-=temps;
		
		if( timer < 0)
		{
			exp.add( new ExplosionEnnemi( (int)x+img.getIconWidth()/2, (int)y-img.getIconHeight()/2) );
			SonManager.getSonManager().jouerExplosionBig();
			explose();
		}
		
		// SI mort on s'occupe des points
		if( ennergie <=0 && it.size() == 0 )
			addPoints();
	}	
	
	
	/**
	 * Dessine
	 */
	public void shape(Graphics graphics) 
	{
		super.shape(graphics);
		

		BufferedImage imageRot= new BufferedImage( img2.getImage().getHeight(null), img2.getImage().getWidth(null), BufferedImage.TYPE_INT_ARGB);

        BufferedImage bufferedImage = new BufferedImage(img2.getImage().getWidth(null), img2.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);
        Graphics g = bufferedImage.createGraphics();
        g.drawImage(img2.getImage(), 0, 0, null);
        g.dispose();
		 
		AffineTransform tx = new AffineTransform();
		tx.rotate((Math.toRadians(rotation)),( img2.getImage().getWidth(null) /2),( img2.getImage().getHeight(null) )/2);
		AffineTransformOp op = new AffineTransformOp(tx,  AffineTransformOp.TYPE_BILINEAR);
		op.filter(bufferedImage, imageRot);
		 
		graphics.drawImage( imageRot, (int)x, (int)y,null);
		
		// POLICE
		graphics.setColor( Color.white );
		graphics.setFont(new Font(Font.MONOSPACED, Font.BOLD, 10));		

		// CONTINUES
		graphics.drawString(Integer.toString( (int)timer/1000 ), (int)x+32, (int)y+37);
				
	}	
	
}