package elements.Ennemis;

import java.awt.Graphics;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;
import controles.Environnement;
import controles.NiveauManager;
import controles.SonManager;
import elements.Explosions.ExplosionEnnemi;
import elements.Tirs.Bombe;
import elements.Tirs.TirEnnemi;


/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class TankMed extends Ennemi
{
	private static final long serialVersionUID = 1L;
	
	private int tempsDeplacement;
	private int tempsDeplacementRestant;
	
	
	private float rotation;
	private float angle;
	
	private int vitesseAngulaire;
	
	private ImageIcon img2;

	
	/**
	 * Constructeur
	 * @param xx x
	 * @param yy y
	 * @param angleDept angle de depart
	 * @param angle angle de deplacement
	 * @param tempsDepl temps du deplacement
	 * @param b bonus
	 */
	public TankMed(int xx,int yy, int vitesse, int vitesseAng, int b, int tempsDepl)
	{
		super(xx,yy,vitesse,0,1200,300,2,b,200,800,1);
		
		vitesseAngulaire = vitesseAng;
		
		angle = 90;
		
		tempsDeplacement 		= tempsDepl;
		tempsDeplacementRestant = tempsDeplacement;
		
		img  = imgLoad.getEnnemi(10);
		img2 = imgLoad.getEnnemi(11);		
	}

	
	/**
	 * Recupre la position X du tir
	 * @return position X du tir
	 */	
	public int getPointTirX()
	{
		return (int) (Math.cos( Math.toRadians(rotation+90) )*44 + x + 14);
	}
	
	
	/**
	 * Recupre la position Y du tir
	 * @return position Y du tir
	 */
	public int getPointTirY()
	{
		return (int)(Math.sin( Math.toRadians(rotation+90) )*44 + y + 20);
	}	

	
	/**
	 * Recupre la vitesse en X du tir
	 * @return vitesse en X du tir
	 */	
	public int getPointTirVitesseX()
	{
		return (int) (Math.cos( Math.toRadians(rotation+90) )*(150 + vitesseX));
	}
	
	
	/**
	 * Recupre la vitesse en Y du tir
	 * @return vitesse en Y du tir
	 */
	public int getPointTirVitesseY()
	{
		return (int)(Math.sin( Math.toRadians(rotation+90) )*(150 + vitesseY));
	}
	
	
	public void gestionTirs()
	{
		if( pretTirer() )
		{
			tirs.add( new TirEnnemi( getPointTirX(), getPointTirY() , getPointTirVitesseX(), getPointTirVitesseY() ) );
			tirOk();
		}
	}	

	
	/**
	 * Recupre la forme de l'ennemi
	 */	
	public Area getArea()
	{
		return new Area( new Rectangle2D.Double(x,y,getWidth(),getHeight()) );
	}	
	
	
	public void collision( Bombe b ){}
	public boolean isPhysique(){return false;}
	
	
	
	/**
	 * Rafraichi les coordonnes
	 */
	public void refresh(long temps) 
	{
		if( tempsDeplacementRestant > 0)
		{
			if( (vitesseAngulaire < 0 && angle > 0 ) || ( vitesseAngulaire > 0 && angle < 180  ) )
				angle += (vitesseAngulaire * temps) / 1000.0;
				
			x += Math.cos( Math.toRadians(angle) ) * (vitesseX * temps / 1000.0);
			y += Math.sin( Math.toRadians(angle) ) * (vitesseX * temps / 1000.0);
			
			tempsDeplacementRestant-=temps;
		}
		
		
		if( y > 0)
		{
			refreshTirs( temps );
		}
		
		//MAJ POSITIONS
		y += (NiveauManager.getNiveauManager().getNiveau().getScrolling() * temps)/1000.0;

		//GESTION DE LA VISEE
		int valeurVitesseX = (int)x - Environnement.getEnvironnement().joueur.getX();
		int valeurVitesseY = (int)y - Environnement.getEnvironnement().joueur.getY();

		rotation = (int)Math.toDegrees( Math.acos(  Math.abs(valeurVitesseX)/(Math.sqrt( valeurVitesseX*valeurVitesseX + valeurVitesseY*valeurVitesseY)) ) );
		
		if( valeurVitesseX < 0 && valeurVitesseY < 0)
			rotation = rotation + 180;
		else if( valeurVitesseX < 0 && valeurVitesseY >= 0)
			rotation = 180 - rotation;
		else if( valeurVitesseX >= 0 && valeurVitesseY < 0)
			rotation = -rotation;
			
		rotation+=90;
		
		if( ennergie <= 0 )
		{
			SonManager.getSonManager().jouerExplosionBig();
			
			// Creer une explosion
			exp.add( new ExplosionEnnemi(getX() + getWidth()/2 ,getY() - getHeight()/2) );
		}
	}	
	
	
	/**
	 * Dessine
	 */
	public void shape(Graphics graphics) 
	{
		// AFFICHAGE BOTTOM
		
		
	
		//Cree un buffer vide
		BufferedImage imageRot= new BufferedImage( img.getImage().getWidth(null), img.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);

		// Copie l'image dans un buffer
        BufferedImage bufferedImage = new BufferedImage(img.getImage().getWidth(null), img.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);
        Graphics g = bufferedImage.createGraphics();
        g.drawImage(img.getImage(), 0, 0, null);
        g.dispose();
       
        // On tourne l'image dans le buffer
		AffineTransform tx = new AffineTransform();
		tx.rotate((Math.toRadians(angle-90)),( img.getImage().getWidth(null) /2),( img.getImage().getHeight(null) )/2);
		AffineTransformOp op = new AffineTransformOp(tx,  AffineTransformOp.TYPE_BILINEAR);
	
		// On met l'image tourne dans ImageRot
		op.filter(bufferedImage, imageRot);
		 
		// On affiche
		graphics.drawImage( imageRot, (int)x-16, (int)y-8,null);
		

		
		// FLASH

		if( hit && Environnement.getEnvironnement().effects == 1)
		{		
			//Cree un buffer vide
			imageRot= new BufferedImage( img.getImage().getWidth(null), img.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);
	
			// Copie l'image dans un buffer
	        bufferedImage = new BufferedImage(img.getImage().getWidth(null), img.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);
	        g = bufferedImage.createGraphics();
	        g.drawImage(img.getImage(), 0, 0, null);
	        g.dispose();
	        
				g.drawImage(img.getImage(), 0, 0, null);
				
				for( int i=0; i<img.getIconWidth();i++)
					for( int j=0; j<img.getIconHeight();j++)
					{
						int rgb = bufferedImage.getRGB(i,j);
						int alpha = (rgb >>24 ) & 0xFF;
	
						if( alpha > 0 )
							bufferedImage.setRGB(i, j, ( (alpha/5)<<24)+(255<<16)+(255<<8)+255 );
					}
				hit = false;
	    
	
	        // On tourne l'image dans le buffer
			tx = new AffineTransform();
			tx.rotate((Math.toRadians(angle-90)),( img.getImage().getWidth(null) /2),( img.getImage().getHeight(null) )/2);
			op = new AffineTransformOp(tx,  AffineTransformOp.TYPE_BILINEAR);
		
			// On met l'image tourne dans ImageRot
			op.filter(bufferedImage, imageRot);
			 
			// On affiche
			graphics.drawImage( imageRot, (int)x-16, (int)y-8,null);		
		}    	
		
		
		
		
		
		
		// AFFICHE TOP
		
		//Cree un buffer vide
		imageRot= new BufferedImage( img2.getImage().getWidth(null), img2.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);

		// Copie l'image dans un buffer
        bufferedImage = new BufferedImage(img2.getImage().getWidth(null), img2.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);
        g = bufferedImage.createGraphics();
        g.drawImage(img2.getImage(), 0, 0, null);
        g.dispose();

        // On tourne l'image dans le buffer
		tx = new AffineTransform();
		tx.rotate((Math.toRadians(rotation)),( img2.getImage().getWidth(null) /2),( img2.getImage().getHeight(null) )/2);
		op = new AffineTransformOp(tx,  AffineTransformOp.TYPE_BILINEAR);
	
		// On met l'image tourne dans ImageRot
		op.filter(bufferedImage, imageRot);
		 
		// On affiche
		graphics.drawImage( imageRot, (int)x -23, (int)y - 18,null);
	}	
	
}