package elements.Ennemis;

import java.awt.Graphics;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;
import controles.Environnement;
import controles.NiveauManager;
import controles.SonManager;
import elements.Explosions.ExplosionEnnemi;
import elements.Tirs.Bombe;
import elements.Tirs.TirEnnemi;


/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class TankSmall extends Ennemi
{
	private static final long serialVersionUID = 1L;
	
	private int tempsDeplacement;
	private int tempsDeplacementRestant;
	
	
	private int rotation;
	
	private ImageIcon img2;

	
	/**
	 * Constructeur
	 * @param xx x
	 * @param yy y
	 * @param angleDept angle de depart
	 * @param angle angle de deplacement
	 * @param tempsDepl temps du deplacement
	 * @param b bonus
	 */
	public TankSmall(int xx,int yy, int vx, int vy, int b, int tempsDepl)
	{
		super(xx,yy,vx,vy,650,200,2,b,200,2000,10);
		
		tempsDeplacement 		= tempsDepl;
		tempsDeplacementRestant = tempsDeplacement;
		
		img  = imgLoad.getEnnemi(8);
		img2 = imgLoad.getEnnemi(9);		
	}

	
	/**
	 * Recupre la position X du tir
	 * @return position X du tir
	 */	
	public int getPointTirX()
	{
		return (int) (Math.cos( Math.toRadians(rotation+90) )*26 + x + 5);
	}
	
	
	/**
	 * Recupre la position Y du tir
	 * @return position Y du tir
	 */
	public int getPointTirY()
	{
		return (int)(Math.sin( Math.toRadians(rotation+90) )*26 + y + 12);
	}	

	
	/**
	 * Recupre la vitesse en X du tir
	 * @return vitesse en X du tir
	 */	
	public int getPointTirVitesseX()
	{
		return (int) (Math.cos( Math.toRadians(rotation+90) )*(150 + vitesseX));
	}
	
	
	/**
	 * Recupre la vitesse en Y du tir
	 * @return vitesse en Y du tir
	 */
	public int getPointTirVitesseY()
	{
		return (int)(Math.sin( Math.toRadians(rotation+90) )*(150 + vitesseY));
	}
	
	
	public void gestionTirs()
	{
		if( pretTirer() )
		{
			tirs.add( new TirEnnemi( getPointTirX(), getPointTirY() , getPointTirVitesseX(), getPointTirVitesseY() ) );
			tirOk();
		}
	}	

	
	/**
	 * Recupre la forme de l'ennemi
	 */	
	public Area getArea()
	{
		return new Area( new Rectangle2D.Double(x,y,getWidth(),getHeight()) );
	}	
	
	
	public void collision( Bombe b ){}
	public boolean isPhysique(){return false;}
	
	
	
	/**
	 * Rafraichi les coordonnes
	 */
	public void refresh(long temps) 
	{
		if( tempsDeplacementRestant > 0)
		{
			if( tempsDeplacementRestant < tempsDeplacement/3)
			{
				y += (vitesseY*temps)/1000.0;
				x += (vitesseX*temps)/1000.0;
			}
			else
			{
				y += (vitesseY*temps*(tempsDeplacement/tempsDeplacementRestant))/1000.0;
				x += (vitesseX*temps*(tempsDeplacement/tempsDeplacementRestant))/1000.0;	
			}
			
			tempsDeplacementRestant-=temps;
		}
		
		
		if( y > 0)
		{
			refreshTirs( temps );
		}
		
		//MAJ POSITIONS
		y += (NiveauManager.getNiveauManager().getNiveau().getScrolling() * temps)/1000.0;

		//GESTION DEPLACEMENT
		int valeurVitesseX = (int)x - Environnement.getEnvironnement().joueur.getX();
		int valeurVitesseY = (int)y - Environnement.getEnvironnement().joueur.getY();

		rotation = (int)Math.toDegrees( Math.acos(  Math.abs(valeurVitesseX)/(Math.sqrt( valeurVitesseX*valeurVitesseX + valeurVitesseY*valeurVitesseY)) ) );
		
		if( valeurVitesseX < 0 && valeurVitesseY < 0)
			rotation = rotation + 180;
		else if( valeurVitesseX < 0 && valeurVitesseY >= 0)
			rotation = 180 - rotation;
		else if( valeurVitesseX >= 0 && valeurVitesseY < 0)
			rotation = -rotation;
			
		rotation+=90;
		
		if( ennergie <= 0 )
		{
			SonManager.getSonManager().jouerExplosionBig();
			
			// Creer une explosion
			exp.add( new ExplosionEnnemi(getX() + getWidth()/2 ,getY() - getHeight()/2) );
		}
	}	
	
	
	/**
	 * Dessine
	 */
	public void shape(Graphics graphics) 
	{
		super.shape(graphics);
		
		//Cree un buffer vide
		BufferedImage imageRot= new BufferedImage( img2.getImage().getWidth(null), img2.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);

		// Copie l'image dans un buffer
        BufferedImage bufferedImage = new BufferedImage(img2.getImage().getWidth(null), img2.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);
        Graphics g = bufferedImage.createGraphics();
        g.drawImage(img2.getImage(), 0, 0, null);
        g.dispose();

        // On tourne l'image dans le buffer
		AffineTransform tx = new AffineTransform();
		tx.rotate((Math.toRadians(rotation)),( img2.getImage().getWidth(null) /2),( img2.getImage().getHeight(null) )/2);
		AffineTransformOp op = new AffineTransformOp(tx,  AffineTransformOp.TYPE_BILINEAR);
	
		// On met l'image tourne dans ImageRot
		op.filter(bufferedImage, imageRot);
		 
		// On affiche
		graphics.drawImage( imageRot, (int)x -15, (int)y - 12,null);
	}	
	
}