package elements.Tirs;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Vector;
import controles.Environnement;
import controles.ImageManager;
import elements.Dessinable;
import elements.Explosions.Explosion;
import elements.Explosions.ExplosionLaser;

/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class Laser extends Dessinable
{
	private static final long serialVersionUID = 1L;
	
	long tempTir;
	long tempsAnimation;
	
	long tempsEntreDeuxFrames;
	
	int maxY;
	int decalage;
	

	Vector<Explosion> exp;
	
	ImageManager imgLoad;

	/**
	 * Constructeur
	 * @param xx x
	 * @param yy y
	 * @param vx Vitesse X
	 * @param vy Vitesse Y
	 * @param l niveau du tir
	 * @param il image loader
	 */
	public Laser(int xx,int yy)
	{
		super(xx,yy);
		
		imgLoad 	= ImageManager.getImagemanager();

		maxY 		= 0;
		img 		= imgLoad.getJoueurLaser( 0 );
		
		exp		   	= new Vector<Explosion>(0);
	}
	
	
	
	/**
	 * Force du laser
	 * @return
	 */
	public int getForce()
	{
		return (int)((Environnement.getEnvironnement().LVL+3)*0.07*tempsEntreDeuxFrames);
	}

	
	/**
	 * recupre le temps d'imunit au laser
	 * @return
	 */
	public int getTempsImunise()
	{
		return 0;
	}
	
	
	/**
	 * Taille du laser
	 */
	public int getWidth()
	{
		return (8* (int)((Environnement.getEnvironnement().LVL+2)/2) )*(Environnement.getEnvironnement().LVL+1)+1;
	}
	
	
	/**
	 * Taille du laser
	 */
	public int getHeight()
	{
		return (int)y - maxY;
	}
	
	
	/**
	 * Arrete le laser a la premiere collision
	 * @param my
	 */
	public void setMaxY( int my )
	{
			maxY = my;
	}
	
	
	/**
	 * Recupre l'air du laser pour voir si il y a effectivement une collision
	 * @return
	 */
	public Area getArea()
	{
		return new Area( new Rectangle2D.Double( (int)x - getWidth()/2 + 30 , (int)maxY, getWidth(), getHeight()));
	}
	
	
	/**
	 * rafraichi posotions
	 */
	public void refresh(long temps, int xx, int yy) 
	{
		x = xx;
		y = yy;
		
		tempsEntreDeuxFrames = temps;
		
		decalage += temps;
		
		if( decalage > 124 )
			decalage -= 124;
		
		tempsAnimation+=temps;
		
		if( tempsAnimation > 100 )
			tempsAnimation -=100;
		
		/**
		 * refresh des explosions
		 */
		for( int i=0; i< exp.size() ; i++)
		{
			if( exp.get(i).isDead() || maxY == 0)
			{
				exp.remove(i);
				i--;
			}
			else
			{
				exp.get(i).refresh(temps);
				exp.get(i).setX(( int)x + 29 );
				exp.get(i).setY(  maxY - 65  );
			}
		}
		
		if( maxY > 0 && exp.size() < 5)
				exp.add( new ExplosionLaser((int)x + 29  ,maxY - 65) );
	}		

	
	
	/**
	 * redessine 
	 */
	public void shape(Graphics graphics) 
	{
		// CREATION DU BUFFER
        BufferedImage bufferedImage = new BufferedImage( getWidth()+1 , 124, BufferedImage.TYPE_INT_ARGB );
        Graphics g 					= bufferedImage.createGraphics();
        g.drawImage( imgLoad.getJoueurLaser(Environnement.getEnvironnement().LVL).getImage(), 0, 0, null );
        g.dispose();
        
        // CAST
		Graphics2D g2d = (Graphics2D)graphics;
		
		// TEXTURE
		g2d.setPaint(new TexturePaint(bufferedImage, new Rectangle((int)x + 30 - getWidth()/2	, -decalage ,getWidth(),124)));
	
		// CALCUL DES ZONES ARRONDIES
		Area cercleBas;
		Area containerBas;
		Area cercleHaut;
		Area containerHaut;
		
		int espacement = 0;
		if( (int)(getHeight() - getWidth()*5.0/4.0) < 0 )
			espacement = (int)(getHeight() - getWidth()*5.0/4.0);

	
		/* GESTION DES TIRS RAPPROCHES */
		if( espacement < 0 )
		{
			cercleHaut 		= new Area( new Ellipse2D.Double  	( (int)x - getWidth()/2 + 30 , (int)maxY	, getWidth(), getHeight()  ) );
		}
		else
		{
			cercleHaut 		= new Area( new Ellipse2D.Double  	( (int)x - getWidth()/2 + 30 , (int)y - getHeight() 	, getWidth(), getWidth()/2 ) );
			containerHaut 	= new Area( new Rectangle2D.Double	( (int)x - getWidth()/2 + 30 , (int)y - getHeight()     , getWidth(), getWidth()/4 ) );
			cercleBas 		= new Area( new Ellipse2D.Double  	( (int)x - getWidth()/2 + 30 , (int)y - getWidth()*2 	, getWidth(), getWidth()*2 ) );
			containerBas 	= new Area( new Rectangle2D.Double	( (int)x - getWidth()/2 + 30 , (int)y - getWidth()   	, getWidth(), getWidth()*2 ) );	
			cercleBas.intersect(containerBas);
			cercleHaut.intersect(containerHaut);
			g2d.fill( cercleBas );
		}
		
		// GERE LES EFFETS
		if(maxY>0)
		{
			g2d.fill( cercleHaut );
			
			if( getHeight() > 0)
				g2d.fillRect( (int)x - getWidth()/2 + 30 , (int)maxY + getWidth()/4, getWidth(), (int)(getHeight() - getWidth()*1.25 + 1) );	
			
			for( int i=0; i< exp.size() ; i++)
				exp.get(i).shape(g2d);
		}
		else
			g2d.fillRect( (int)x - getWidth()/2 + 30 , (int)y - getHeight()				, getWidth(), getHeight() - getWidth() );	
	}	
	
	
	/**
	 * permet de savoir si c'est un laser simple ou un laserBombe
	 */
	public String toString()
	{
		return "laser";
	}
}
