package jeu;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.image.BufferStrategy;
import java.io.FileInputStream;
import java.util.Properties;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;

import controles.Clavier;
import controles.ScreenManager;
import controles.SonManager;
import menus.*;


/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class BlackArmada extends Canvas
{

	private static final long serialVersionUID = 1L;
	private BufferStrategy strategy;
    private JFrame frame;
	private ImageIcon icon;
	
	private Menu menuCourrant;
	
	private SonManager son;
	private Clavier clavier;
    private ScreenManager sm;
    private DisplayMode dm;
    
    private int fullScreen;
    
	private Partie p;
	
    /**
     * Constructeur
     */
	public BlackArmada() 
	{
		// MENUS ET CONTROLES
		clavier 		= Clavier.getClavier();
    	sm 				= new ScreenManager();
    	
		menuCourrant 	= new MenuPub();	
		
		// Sons
		son = SonManager.getSonManager();
		son.setMusiqueEnCours( son.MusiqueMenu );
		
		// fen�tre du jeu
		frame = new JFrame("Black Armada");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Images
		icon = new ImageIcon( Toolkit.getDefaultToolkit().getImage("Images/Logo/BA-16.png"));
		frame.setIconImage( icon.getImage() );
		
		// Fenetre
		frame.setResizable(false);
		frame.setUndecorated(false);

    	// Options
    	readOptions();
    	
    	// FullScreen
    	if( fullScreen == 1 )
    		goFullScreen();			
		
		//contenu de la fen�tre
		JPanel panel = (JPanel) frame.getContentPane();
		panel.setPreferredSize(new Dimension(1024, 768));
		panel.setLayout(null);
		panel.add(this);
		
		// zone d'affichage
		setBounds(0, 0, 1024, 768);
		setIgnoreRepaint(true);	
		
		// fenetre
    	frame.pack(); 
    	if( fullScreen == 0 ) 	
		frame.setLocationRelativeTo(frame.getParent());    
		frame.setVisible(true);
    	
	    // Affichage
	    setIgnoreRepaint( true );  
	    createBufferStrategy( 2 ); 
	    strategy = getBufferStrategy(); 
	     
	    // Clavier
		requestFocus();
		addKeyListener(Clavier.getClavier());
	
		// Lancer Menu
		run();
	}   
	
	/**
	 * Lire options
	 */
	public void readOptions()
	{
		Properties lecteur = new Properties();
		FileInputStream fichier;
		
		try
		{
			fichier = new FileInputStream("Data/options.dat");
			lecteur.load(fichier);
			fichier.close();
			fullScreen = Integer.parseInt( lecteur.getProperty("fullScreen") );	
		}
		catch( Exception e)
		{
			fullScreen = 0;
		}
	}	
	
	
	/**
	 * Aller FullScreen
	 */
	private void goFullScreen()
	{
		setCursor(Toolkit.getDefaultToolkit().createCustomCursor( Toolkit.getDefaultToolkit().getImage("Images/Divers/mouse.png") , new Point(16, 16), "mouse"));
		
		dm = new DisplayMode( 1024, 768, 32,  DisplayMode.REFRESH_RATE_UNKNOWN);
		sm.setFullScreen(frame, dm);   		
	}
	
	
	/**
	 * Aller en mode Fenetrer
	 */
	private void goFenetre()
	{
		sm.restoreFullScreen();
		frame.setLocationRelativeTo(getParent());
		frame.setUndecorated(false);
	}

	
	/**
	 * Lancer le jeu
	 */
	private void run() 
	{
		long temps = System.currentTimeMillis();
		long tempsPrecedent = System.currentTimeMillis();

		while (true) 
		{
			Graphics2D graphics2D = (Graphics2D) strategy.getDrawGraphics();
			
			
			//////// --------- FPS ------- /////////////
			temps = System.currentTimeMillis() - tempsPrecedent;
			tempsPrecedent = System.currentTimeMillis();
			
			// ON DESSINE LE MENU
			menuCourrant.refresh( temps );
			menuCourrant.shape( graphics2D );
			

			// GESTION DES MENUS
			if( menuCourrant.getTypeDeMenu() == "MenuPrincipal" )
			{
				
				if( ( (MenuPrincipal) menuCourrant ).getGoJeu() )
				{
					//Reset Clavier
					clavier.entrerOk();
					
					// Sons
					son.jouerBeep2();
					son.stopMusique();
					
					// Vide la memoire
					menuCourrant = null;
					System.gc();
					
					//Demarer jeu
					p = new Partie();
					p.run(strategy);
					
					tempsPrecedent = System.currentTimeMillis();
					
					if( p.goMenuFinJeu() )
					{
						menuCourrant = new MenuFinJeu();
					}
					else
					{
						menuCourrant = new MenuPrincipal();
						son.setMusiqueEnCours( son.MusiqueMenu );
						son.jouerMusique();
					}
					
					// Vide la memoire
					p = null;
					System.gc();
				}				
				else if( ( (MenuPrincipal) menuCourrant ).getGoOption() )
				{
					//Reset Clavier
					clavier.entrerOk();	
					
					// Sons
					son.jouerBeep2();
					
					//Entre dans les options
					menuCourrant = new MenuOptions();
				}
				else if( ( (MenuPrincipal) menuCourrant ).getGoScores() )
				{
					//Reset Clavier
					clavier.entrerOk();	
					
					// Sons
					son.jouerBeep2();
					
					menuCourrant = new MenuScore();
				}				
				else if( ( (MenuPrincipal) menuCourrant ).getGoAPropos() )
				{
					//Reset Clavier					
					clavier.entrerOk();				

					// Sons	
					son.jouerBeep2();
					
					// Entre dans les a propos
					menuCourrant = new MenuAPropos();			

				}
				else if( ( (MenuPrincipal) menuCourrant ).visitWebsite() )
				{
					//Reset Clavier					
					clavier.entrerOk();		
					
					Runtime r = Runtime.getRuntime();
					try
					{
			    		if ( System.getProperty("file.separator").equals( "/" ) ) 
			    			r.exec("firefox http://jerome-baudoux.com/BlackArmada/menu-en-black-armada-by-the-game.html");			    			
			    		else
			    			r.exec("cmd /c start http://jerome-baudoux.com/BlackArmada/menu-en-black-armada-by-the-game.html");
						
						if( fullScreen == 1 )
						{
							// Sons
							son.jouerBeep2();
							son.stopMusique();
							
							//Retour en mode fenetr� pour pas faire bugger windows
							if( fullScreen == 1)
								goFenetre();
							
							// Quitte le jeu
							System.exit(0);
						}
					}
					catch (Exception ex){} 		

				}
				else if( ( (MenuPrincipal) menuCourrant ).getExit() )
				{
					// Sons
					son.jouerBeep2();
					son.stopMusique();
					
					//Retour en mode fenetr� pour pas faire bugger windows
					if( fullScreen == 1)
						goFenetre();
					
					// Quitte le jeu
					System.exit(0);
				}
				
			}
			else if( menuCourrant.getTypeDeMenu() == "MenuOptions" )
			{
				if( ( (MenuOptions) menuCourrant ).getGoMenuPrincipal() )
				{
					//Reset Claiver
					clavier.entrerOk();
					clavier.ESCOk();
					
					// Sons
					son.jouerBeep2();
					
					// Retourne dans le menu principal
					menuCourrant = null;
					System.gc();
					menuCourrant = new MenuPrincipal();

				}
				else if( ( (MenuOptions) menuCourrant ).getGoMenuClavier() )
				{
					//Reset Claiver
					clavier.entrerOk();
					clavier.ESCOk();
					
					// Sons
					son.jouerBeep2();
					
					// Retourne dans le menu principal
					menuCourrant = null;
					System.gc();
					menuCourrant = new MenuClavier();

				}
			}
			else if( menuCourrant.getTypeDeMenu() == "MenuClavier" )
			{
				if( ( (MenuClavier) menuCourrant ).getGoMenuOption() )
				{
					//Reset Claiver
					clavier.entrerOk();
					clavier.ESCOk();
					
					// Sons
					son.jouerBeep2();
					
					// Retourne dans le menu principal
					menuCourrant = null;
					System.gc();
					menuCourrant = new MenuOptions();

				}
			}			
			else if( menuCourrant.getTypeDeMenu() == "MenuAPropos" )
			{
				if( ( (MenuAPropos) menuCourrant ).getGoMenuPrincipal() )
				{
					//Reset Claiver
					clavier.entrerOk();
					clavier.ESCOk();
					
					// Sons
					son.jouerBeep2();
					
					// Retourne dans le menu principal
					menuCourrant = null;
					System.gc();
					menuCourrant = new MenuPrincipal();
				}
			}
			else if( menuCourrant.getTypeDeMenu() == "MenuScore" )
			{
				if( ( (MenuScore) menuCourrant ).getGoMenuPrincipal() )
				{
					//Reset Claiver
					clavier.entrerOk();
					clavier.ESCOk();
					
					// Sons
					son.jouerBeep2();
					
					// Retourne dans le menu principal
					menuCourrant = null;
					System.gc();
					menuCourrant = new MenuPrincipal();
				}
			}				
			else if( menuCourrant.getTypeDeMenu() == "MenuPub" )
			{
				if( ( (MenuPub) menuCourrant ).isDone() )
				{
					//Reset Claiver
					clavier.entrerOk();
					clavier.ESCOk();

					// Sons			
					son.setMusiqueEnCours( son.MusiqueMenu );
					son.jouerMusique();
					
					// Retourne dans le menu principal	
					menuCourrant = null;
					System.gc();
					menuCourrant = new MenuPrincipal();	
					
					// On vide la memoire de ce qui ne sert a rien
					System.gc();
				}
			}
			else if( menuCourrant.getTypeDeMenu() == "MenuFinJeu" )
			{
				if( ( (MenuFinJeu) menuCourrant ).isDone() )
				{
					//Reset Claiver
					clavier.entrerOk();
					clavier.ESCOk();

					// Sons
					son.setMusiqueEnCours( son.MusiqueMenu );
					son.jouerMusique();
					
					// Retourne dans le menu principal
					menuCourrant = null;
					System.gc();
					menuCourrant = new MenuPrincipal();	
				}
			}			
			
			// DESSINE L'ECRAN
			graphics2D.dispose();
			strategy.show();	
			
			// PAUSE POUR LE PROSS
			try 
			{
				Thread.sleep(10);
			} 
			catch (InterruptedException e) 
			{
				e.printStackTrace();
			}			
		}
	}
	

	/**
	 * MAIN
	 * @param args
	 */
	public static void main(String[] args) 
	{
		new BlackArmada();
	}

}
