package jeu;

import java.awt.AlphaComposite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferStrategy;
import java.util.Collections;
import controles.*;
import elements.*;
import elements.Explosions.*;

/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class Partie 
{
	// ENVIRONNEMENT ET ELEMENTS
	private Environnement 	env;
	private Clavier 		clavier;	
	private NiveauManager 	niveau;
	private SonManager 		son;


	/**
	 * Constucteur
	 */
	public Partie()
	{
		Environnement.restoreEnvironnement();
		
		Environnement.getEnvironnement().joueur 	= new Joueur();
		
		NiveauManager.RestoreNiveauManager();
		niveau  = NiveauManager.getNiveauManager();

		clavier = Clavier.getClavier();
		son		= SonManager.getSonManager();
		env 	= Environnement.getEnvironnement();	
	}
	
	public boolean goMenuFinJeu()
	{
		return env.jeuFini;
	}
	
	
	/**
	 * Boucle de Jeu
	 * 
	 * @param strategy
	 */
	public void run(BufferStrategy strategy)
	{
		long temps = System.currentTimeMillis();
		long tempsPrecedent = System.currentTimeMillis();

		// Lance la musique
		//	son.setMusiqueEnCours( son.musiqueJeu );
		//	son.jouerMusique();
		
		while ( ! env.goToMenu() ) 
		{
			
			if( env.isPlay())
			{
				/////// --------- PAUSE --------- ////////////
				if( clavier.isPause() )
				{
					clavier.pauseOk();					
					if( env.pause == Environnement.MODE_JEU )
					{				
						env.pause = 1;
						son.pauseMusique();
						son.pauseWarning();
						son.stopLaser();
					}
					else if( env.pause == Environnement.MODE_PAUSE )
					{
						env.pause = 0;
						son.jouerMusique();					
					}
				}
				
				// Mise en pause menu de choix
				if( clavier.isESC() && env.pause == Environnement.MODE_JEU )
				{
					clavier.ESCOk();							
					env.pause = 2;
					son.pauseMusique();
					son.pauseWarning();
					son.stopLaser();
				}
				
				// recupre le statut de fin de menu re choix
				if( env.pause == Environnement.RETOUR_AU_JEU)
				{
					env.pause = 0;
					son.jouerMusique();						
				}
				
				//TIRS
				if( env.isPlay() && ! env.isPause())
				{
					Environnement.getEnvironnement().joueur.gereTirsEtLaser(temps);
				}
			}
			
			
			//////// --------- FPS ------- /////////////
			
			temps = System.currentTimeMillis() - tempsPrecedent;
			tempsPrecedent = System.currentTimeMillis();
			
			// permet de 'rammer' si le fps est trop bas
			if( temps > 90 )
				temps = 90;
			
			
			// TRI DES VECTORS
			Collections.sort( env.tirsJoueur );
			Collections.sort( env.ennemis );
			
			
			//////// --------- AFFICHAGE ------- /////////////	
			
			// Refresh
			collisions();				
			refresh(temps);
			
			Collections.sort( env.ennemis );
		

			// contexte graphique		
			Graphics2D graphics2D = (Graphics2D) strategy.getDrawGraphics();	
			
			// initialisation du mode phraphique
			if( env.antiAliasing == 0)
				graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
			else
				graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			
			if( env.antiAliasingText == 0)
				graphics2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);	
			else
				graphics2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);	
			
			graphics2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
			graphics2D.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_SPEED);					
			
			// Shape
			shape(graphics2D);
				

			// affichage dans la fentre
			graphics2D.dispose();
			strategy.show();
			
			try 
			{
				Thread.sleep(5);
			} 
			catch (InterruptedException e) 
			{
				e.printStackTrace();
			}	
		}
		
		son.stopMusique();		
	}
	
	
	/**
	 * Gere les Collsions 
	 */
	private void collisions()
	{
		if( env.isPlay() && ! env.isPause() )
		{
			// Collision Laser
			if( env.laser != null )
			{
				boolean ok = false;
				for( int j=0; j<env.ennemis.size(); j++)
					if( env.ennemis.get(j).collision(env.laser) )
						ok = true;
				
				
				// Si le laser n'a pas eu de collision on l'agrandie et on regarde a nouveau les collision
				if( ! ok )
				{
					env.laser.setMaxY(0);
					
					for( int j=0; j<env.ennemis.size(); j++)
						if( env.ennemis.get(j).collision(env.laser) )
							ok = true;
					
				}
				
				// Si le laser n'a pas eu de collision, il peut aller jusqu'a la fin de l'ecran
				if( ! ok )
					env.laser.setMaxY(0);
				
				// Collisions entre les tirs et le laserbombe
				if( env.laser.toString().equals("bombe"))
				{
					for( int j=0; j<env.tirsEnnemi.size(); j++)
						env.tirsEnnemi.get(j).collision(env.laser);
				}
			}
			
			// Autres collisions
			for(int i=0; i<env.tirsJoueur.size(); i++)
				for( int j=0; j<env.ennemis.size(); j++)
					env.ennemis.get(j).collision(env.tirsJoueur.get(i));
			
			for(int i=0; i<env.bombes.size(); i++)
				for( int j=0; j<env.ennemis.size(); j++)
					env.ennemis.get(j).collision(env.bombes.get(i));
				
			for(int i=0; i<env.bombes.size(); i++)
				for( int j=0; j<env.tirsEnnemi.size(); j++)
					env.tirsEnnemi.get(j).collision(env.bombes.get(i));
			
			for(int i=0; i<env.items.size(); i++)
				Environnement.getEnvironnement().joueur.collision(env.items.get(i));
			
			for(int i=0; i<env.tirsEnnemi.size(); i++)
				Environnement.getEnvironnement().joueur.collision(env.tirsEnnemi.get(i));
			
			for(int i=0; i<env.ennemis.size(); i++)
				Environnement.getEnvironnement().joueur.collision(env.ennemis.get(i));
		}
	}
	
	
	/**
	 * Rafraichi les positions
	 * 
	 * @param temps
	 */
	private void refresh(long temps)
	{

		Environnement.getEnvironnement().joueur.refresh(temps);			
		niveau.refresh(temps);
		
		if( ! env.isPause())
		{
			// inutile de raffraichir ca si on est en mode evennement
			
			if( env.isPlay() || env.gameState == Environnement.LEVEL_CLEAR )
			{
				
				env.em.refresh(temps);
				
				/// -----------------------
				//
				// Refresh les explosions
				//
				// ------------------------
				
				for(int i=0;i<env.exps.size();i++)
				{
					env.exps.get(i).refresh(temps);
					
					// Suppr les explosions mortes 
					if( env.exps.get(i).isDead())
					{
						env.exps.remove(i);
						i--;
					}			
				}			
				
				/// -----------------------
				//
				// Refresh le Laser
				//
				/// -----------------------
				
				if( env.laser != null )
					env.laser.refresh(temps, Environnement.getEnvironnement().joueur.getX(), Environnement.getEnvironnement().joueur.getY());
			
				/// -----------------------
				//
				// Refresh les tirs JOUEUR
				//
				/// -----------------------
				
				for(int i=0;i<env.tirsJoueur.size();i++)
				{
					env.tirsJoueur.get(i).refresh(temps);
					
					// Suppr les tirs qui depassent l'ecran ou qui sont morts et cree une explosion
					if(env.tirsJoueur.get(i).getY() < -80 || env.tirsJoueur.get(i).getX() < 100 || env.tirsJoueur.get(i).getX()>802 || ! env.tirsJoueur.get(i).isVisible())
					{
						if( ! env.tirsJoueur.get(i).isVisible() )
						{
							env.exps.add( new ExplosionTir(env.tirsJoueur.get(i).getX(),env.tirsJoueur.get(i).getYPrededent()) );	
							son.jouerExplosion();
						}
						
						env.tirsJoueur.remove(i);

						i--;
					}			
				}	
				
				/// -----------------------
				//
				// Refresh les Bombes  JOUEUR
				//
				/// -----------------------
				
				for(int i=0;i<env.bombes.size();i++)
				{
					env.bombes.get(i).refresh(temps);
					
					// Suppr les tirs qui depassent l'ecran ou qui sont morts et cree une explosion
					if( ! env.bombes.get(i).isVisible() )
					{
						env.bombes.remove(i);
						i--;
					}			
				}						
				
				/// ---------------------------
				//
				// Refresh les ennemis
				//
				// ----------------------------
				
				for(int i=0;i<env.ennemis.size();i++)
				{
					env.ennemis.get(i).refresh(temps);
					
					env.exps.addAll( env.ennemis.get(i).getExplosions() );
					env.ennemis.get(i).clearExplosions();
					
					//Si l'ennemi a tu le joueur
					if( env.ennemis.get(i).getATueLeJoueur() )
					{
						env.ennemis.get(i).setATueLeJoueur( false );
						env.exps.add( new ExplosionJoueur(Environnement.getEnvironnement().joueur.getX() + Environnement.getEnvironnement().joueur.getWidth()/2 ,Environnement.getEnvironnement().joueur.getY() + Environnement.getEnvironnement().joueur.getHeight()/2) );
						env.exps.add( new ExplosionEnnemi(Environnement.getEnvironnement().joueur.getX() + Environnement.getEnvironnement().joueur.getWidth()/2 ,Environnement.getEnvironnement().joueur.getY() - Environnement.getEnvironnement().joueur.getHeight()/2) );							
					}
					
					// TIRS DES ENNEMIS
					if( env.ennemis.get(i).getTirs() != null )
					{
						env.tirsEnnemi.addAll( env.ennemis.get(i).getTirs() );
						env.ennemis.get(i).clearTirs();
					}

					// Suppr les ennemis qui depassent l'ecran ou qui sont morts
					if( ( ( env.ennemis.get(i).getY() > 770 || env.ennemis.get(i).getX()<100 || env.ennemis.get(i).getX() > 900 ) &&  env.ennemis.get(i).getType().equals("Ennemi") ) || env.ennemis.get(i).isDead() )
					{
						// SI MORT
						if( env.ennemis.get(i).isDead() )
						{
							
							// SI TUE PAR LE JOUEUR
							if( env.ennemis.get(i).isKilled() )
							{
							
								//Ajoute les points
								env.addPoints(env.ennemis.get(i).getPoints());
								
				
								env.items.addAll( env.ennemis.get(i).getItems() );
								env.ennemis.get(i).clearItems();
							
							}
						}
						env.ennemis.remove(i);
						i--;
					}
					
				}
				
			}
			
			/// ----------------------------
			//
			// Refresh les tirs ENNEMIS
			//
			/// ----------------------------
			
			for(int i=0;i<env.tirsEnnemi.size();i++)
			{
				env.tirsEnnemi.get(i).refresh(temps);
				
				// Suppr les tirs qui depassent l'ecran ou qui sont morts ou bien tous les tirs si le niveau est fini
				if( env.tirsEnnemi.get(i).getY() < -80 || env.tirsEnnemi.get(i).getY() > 770 || env.tirsEnnemi.get(i).getX() < 100 || env.tirsEnnemi.get(i).getX()>802 || ! env.tirsEnnemi.get(i).isVisible() || env.gameState == Environnement.LEVEL_CLEAR )
				{
					if( ! env.tirsEnnemi.get(i).isVisible() && env.tirsEnnemi.get(i).getATueLeJoueur() )
					{
						env.exps.add( new ExplosionJoueur(Environnement.getEnvironnement().joueur.getX() + Environnement.getEnvironnement().joueur.getWidth()/2 ,Environnement.getEnvironnement().joueur.getY() + Environnement.getEnvironnement().joueur.getHeight()/2) );
						env.exps.add( new ExplosionEnnemi(Environnement.getEnvironnement().joueur.getX() + Environnement.getEnvironnement().joueur.getWidth()/2 ,Environnement.getEnvironnement().joueur.getY() - Environnement.getEnvironnement().joueur.getHeight()/2) );					
					}
					
					env.tirsEnnemi.remove(i);
					i--;
				}			
			}		
			
			
			/// ----------------------------
			//
			// Refresh les ITEMS
			//
			/// ----------------------------
			
			for(int i=0;i<env.items.size();i++)
			{
				env.items.get(i).refresh(temps);
				
				// Suppr les tirs qui depassent l'ecran ou qui sont morts
				if( env.items.get(i).getY() > 770 || env.items.get(i).isDead() )
				{
					if( env.items.get(i).isDead() )
					{
						if( env.items.get(i).toString() == "ItemTir")
						{
							son.jouerItemtir();
						}
						else
						{
							son.jouerPoints();
						}

					}
					
					env.items.remove(i);
					i--;
				}			
			}				
			
			/// ----------------------------
			//
			// Refresh l'environnement
			//
			/// ----------------------------
			
				
		}

		env.refresh(temps);
	}
	
	
	/**
	 * Redessine tout
	 * 
	 * @param g
	 */
	private void shape(Graphics g)
	{
		Graphics2D g2d = (Graphics2D)g;
		
		niveau.shapeBackground(g);
	
		for(int i=0;i<env.ennemis.size();i++)
			env.ennemis.get(i).shape(g);		
		
		g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.75f ));
		
		for(int i=0;i<env.tirsJoueur.size();i++)
			env.tirsJoueur.get(i).shape(g2d);
		
		g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f ));
		
		if( env.laser != null )
			env.laser.shape(g);
		
		Environnement.getEnvironnement().joueur.shape(g);
		
		for(int i=0;i<env.items.size();i++)
			env.items.get(i).shape(g);			
		
		for(int i=0;i<env.exps.size();i++)
			env.exps.get(i).shape(g);
		
		for(int i=0;i<env.bombes.size();i++)
			env.bombes.get(i).shape(g);	
		
		for(int i=0;i<env.tirsEnnemi.size();i++)
			env.tirsEnnemi.get(i).shape(g);	
		
		niveau.shapeForeground(g);	
		
		for(int i=0;i<env.ennemis.size();i++)
			env.ennemis.get(i).shapeForeground(g);
		
		/* Dessine les bords noirs
		g.setColor(Color.black);
		g.fillRect(0, 0, 212, 768);			
		g.fillRect(812, 0, 212, 768);	*/
		
		g2d.drawImage( ImageManager.getImagemanager().getImageFond().getImage(),0,0,null);

		env.shape(g);		
	}
}
