package launcher;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class BlackArmadaLauncher extends JFrame
{
	private static final long serialVersionUID = 1L;
	private LauncherOptions options;
	private LauncherKeys 	keys;
	private LauncherScores 	scores;
	private LauncherAbout 	about;

	/**
	 * Constructeur
	 */
	public BlackArmadaLauncher()
	{
		init();
		contains();
	}
	
	/**
	 * Initialisateur
	 */
	private void init()
	{
		try 
		{
		   UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		   SwingUtilities.updateComponentTreeUI(this);
		}
		catch (Exception e) {}

		setTitle("Black Armada Launcher");
		setSize(300,400);
		setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setIconImage( new ImageIcon(   Toolkit.getDefaultToolkit().getImage(this.getClass().getResource(".").getPath().replaceAll("%20", " ")+"../../Images/Logo/BA-16.png")).getImage() );
		setLocationRelativeTo(getParent());
	}
	
	/**
	 * Ajoute les elements
	 */
	private void contains()
	{
		ImageIcon logo = new ImageIcon(Toolkit.getDefaultToolkit().getImage(this.getClass().getResource(".").getPath().replaceAll("%20", " ")+"../../Images/Logo/launcher.png"));
		JPanel panel =  new JPanel();
		
		// BANDEAU
		panel.add( new JLabel( logo ), BorderLayout.NORTH );
		
		// ONGLETS
		options = new LauncherOptions();
		keys 	= new LauncherKeys();
		scores 	= new LauncherScores();
		about 	= new LauncherAbout();
			
		JTabbedPane jtp = new JTabbedPane();
		jtp.setPreferredSize( new Dimension(280,260));
		
		jtp.addTab(options.getTitle()	, options	);
		jtp.addTab(keys.getTitle()		, keys.getJPanel()	);
		jtp.addTab(scores.getTitle()	, scores	);
		jtp.addTab(about.getTitle()		, about		);

		panel.add( jtp, BorderLayout.CENTER );
		
		// BOUTON LANCER
		JButton run = new JButton("SAVE OPTIONS AND RUN THE GAME");
		run.addActionListener( new ActionListener()
		{
		    public void actionPerformed (ActionEvent evt)
		    {
		    	options.writeOptions();
		    	keys.writeKeys();
		    	Thread runGame  = new Thread()
		    	{
		    		public void run()
		    		{
		    			try
		    			{
			    			Runtime runtime = Runtime.getRuntime();
			    			runtime.exec("Black Armada.exe");
		    			}
		    			catch( Exception e ){}

		    		}
		    	};
		    	
		    	runGame.start();
		    	System.exit(0);
			}
		});
		
		panel.add( run, BorderLayout.SOUTH );
		add( panel );
		repaint();
		setVisible(true);
		run.requestFocus();
	}
	


	/**
	 * MAIN
	 * @param arg
	 */
	public static void main( String arg[] )
	{
		new BlackArmadaLauncher();
	}
}
