package niveaux;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import javax.swing.ImageIcon;
import controles.Environnement;
import controles.SonManager;

/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class Niveau0 extends Niveau
{
	public static int PAUSE		= -1;
	public static int DEBUT 	= 0;
	public static int TOURNE 	= 1;
	public static int OUVRE 	= 2;
	public static int LANCE 	= 3;
	public static int AVANCE 	= 4;
	public static int FIN 		= 5;
	
	private ImageIcon spin;
	private ImageIcon boxG;
	private ImageIcon boxD;
	private ImageIcon launch;
	
	private float spinRotation;
	private float spinX;
	private float spinY;
	private float boxGX;
	private float boxGY;
	private float boxDX;
	private float boxDY;
	private float launchX;
	private float launchY;
	
	private float red;
	
	private int temps;
	private int etat;
	
	public Niveau0()
	{
		super(0,0);
		
		etat = PAUSE;
		
		yBackground = -4000 + 768;
		yForeground = -3000 + 768;

		
		background 	= new ImageIcon(Toolkit.getDefaultToolkit().getImage("Images/Intro/fond.png"));
		foreground 	= new ImageIcon(Toolkit.getDefaultToolkit().getImage("Images/Intro/murs.png"));	
		
		spin 		= new ImageIcon(Toolkit.getDefaultToolkit().getImage("Images/Intro/spin.png"));
		boxG 		= new ImageIcon(Toolkit.getDefaultToolkit().getImage("Images/Intro/g.png"));
		boxD 		= new ImageIcon(Toolkit.getDefaultToolkit().getImage("Images/Intro/d.png"));
		launch 		= new ImageIcon(Toolkit.getDefaultToolkit().getImage("Images/Intro/launch.png"));
		
		boxGX		= 414;
		boxGY		= 3600;
		
		boxDX		= 512;
		boxDY		= 3600;
		
		launchX		= 500;
		launchY		= 3000;
		
		spinX 		= boxDX + 40;
		spinY 		= boxDY + 350;
		
		red = 0;
		
		// JOUER MUSIQUE
		SonManager.getSonManager().jouerMusique();
	}
	
	public void refresh( long t )
	{
		if( etat == PAUSE )
		{
			if( temps > 1000 )
			{
				temps = 0;
				
				// JOUER ALARME
				if( SonManager.getSonManager().temp[0] != null )
					SonManager.getSonManager().temp[0].boucle();
				
				etat = DEBUT;
			}
			else
				temps += t;
		}
		else if( etat == DEBUT )
		{
			if( temps > 3000 )
			{
				temps = 0;
				etat = TOURNE;
				
				if( SonManager.getSonManager().temp[1] != null )
					SonManager.getSonManager().temp[1].jouer();
			}
			else
				temps += t;
		}
		if( etat == TOURNE )
		{
			if( temps > 1000 )
			{
				temps = 0;
				etat = OUVRE;
			}
			else
			{
				temps += t;
				spinRotation += t/4f;
				
				if( spinRotation > 360 )
					spinRotation -= 360;
			}
		}
		if( etat == OUVRE )
		{
			if( temps > 2000 )
			{
				temps = 0;
				speedBackground = 400;
				speedForeground = 400;
				Environnement.getEnvironnement().joueur.setVitesseY(-40);
				etat = AVANCE;
			}
			else
				temps += t;
			
			if( temps < 1000 )
			{
				boxGX -= t/10;
				boxDX += t/10;
				spinX += t/10;
			}
		}
		else if( etat == AVANCE )
		{
			speedBackground += t;
			speedForeground += t;
		}
		else if( etat == FIN )
		{
			temps += t;
			
			if( SonManager.getSonManager().temp[0] != null )
				SonManager.getSonManager().temp[0].stop();
		}
		
		if( etat != PAUSE )
		{
			yBackground += (speedBackground*t)/1000.0;
			yForeground += (speedForeground*t)/1000.0;
			
			red += t/2;
	
			
			// BLOQUE -> FIN
			
			if( yBackground > -10 )
			{
				yBackground = 0;
				Environnement.getEnvironnement().joueur.setVitesseY(-1500);
			}
			
			if( Environnement.getEnvironnement().joueur.getY() < - 600 )
				etat = FIN;
		}
	}
	
	public boolean finIntro()
	{
		return etat == FIN && temps > 2000;
	}
	
	public void shapeBackground(Graphics graphics) 
	{
		super.shapeBackground(graphics);
		
		graphics.drawImage(launch.getImage(), 	(int)launchX, 	(int)(launchY + yBackground), null);

	}
	
	public void shapeForeground(Graphics graphics) 
	{
		graphics.drawImage(boxG.getImage(), 	(int)boxGX, 	(int)(boxGY + yBackground)	, null);
		graphics.drawImage(boxD.getImage(), 	(int)boxDX, 	(int)(boxDY + yBackground)	, null);
		
		BufferedImage imageRot= new BufferedImage( spin.getImage().getHeight(null), spin.getImage().getWidth(null), BufferedImage.TYPE_INT_ARGB);
        BufferedImage bufferedImage = new BufferedImage(spin.getImage().getWidth(null), spin.getImage().getHeight(null), BufferedImage.TYPE_INT_ARGB);
        Graphics g = bufferedImage.createGraphics();
        g.drawImage(spin.getImage(), 0, 0, null);
        g.dispose();
		AffineTransform tx = new AffineTransform();
		tx.rotate((Math.toRadians(spinRotation)),( spin.getImage().getWidth(null) /2),( spin.getImage().getHeight(null) )/2);
		AffineTransformOp op = new AffineTransformOp(tx,  AffineTransformOp.TYPE_BILINEAR);
		op.filter(bufferedImage, imageRot);
		graphics.drawImage( imageRot, (int)spinX, (int)(spinY + yBackground),null);
		
		graphics.drawImage( foreground.getImage() ,212, (int)yForeground, null  );
		
		if( red > 510 )
			red -= 510;
		
		if( red > 255 )
		{
			//Fade out lors de la fin du niveau
			graphics.setColor( new Color( 1,0,0, (510 - red)/510f ) );
			graphics.fillRect(212, 0, 600, 768);	
		}
		else
		{
			//Fade out lors de la fin du niveau
			graphics.setColor( new Color( 1,0,0, red/510f ));
			graphics.fillRect(212, 0, 600, 768);
		}
		
		
		if( etat == FIN )
		{
			int alpha = (int)temps*255/2000;
			
			if( alpha > 255 )
				alpha = 255;
			
			//Fade out lors de la fin du niveau
			graphics.setColor( new Color( 1,1,1, alpha ));
			graphics.fillRect(212, 0, 600, 768);
		}
	}	
}
