package controles;

import java.awt.DisplayMode;
import java.awt.Frame;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Window;

/**
 * Cette classe est un gestionnaire d'cran et permet de changer la rsolution
 * de l'cran ou de mettre le mode plein-cran.
 * 
 * @author Julien CHABLE (webmaster@neogamedev.com)
 * @version 1.0 20/06/2004
 */
public class ScreenManager {

    // Notre device (abstraction de la carte graphique ...) graphique sur lequel
    // nous allons effectuer les changements de rsolution, le plein-cran, ...
    private GraphicsDevice gd;

    /**
     * Constructeur par dfaut. Initialise la variable du GraphicsDevice.
     *  
     */
    public ScreenManager() {
        // On rcupre l'environnement graphique du systme d'exploitation,
        // celui-ci va nous permettre de rcuprer un certain nombre d'lments
        // sur la configuration graphique du systme ...
        GraphicsEnvironment ge = GraphicsEnvironment
                .getLocalGraphicsEnvironment();
        // On rcupre le device par dfaut ... vous pouvez rcuprer l'ensemble
        // des devices du systme avec la mthode getScreenDevices(), le centre
        // de l'cran avec getCenterPoint(), ...
        gd = ge.getDefaultScreenDevice();
    }

    /**
     * Passage de la fentre en mode plein cran. Changement de mode graphique (rsolution et profondeur). Regarder la
     * classe DisplayMode pour plus d'informations sur les rsolutions d'cran.
     * 
     * @param window
     *            La fentre  passer en plein-cran.
     * @param displayMode
     *            Le mode graphique  appliquer.
     */
    public void setFullScreen(Frame window, DisplayMode displayMode) {
        if (window == null) return;
        // On ne veut pas des bords de la fentre
        window.setUndecorated(true);
        // La fentre n'est pas redimensionnable
        window.setResizable(false);
        // Passage en mode plein-cran
        if (gd.isFullScreenSupported()) gd.setFullScreenWindow(window);
        // Changement de rsolution
        if (displayMode != null && gd.isDisplayChangeSupported()) {
            try {
                // Affectation du mode graphique au device graphique.
                gd.setDisplayMode(displayMode);
            } catch (Exception e) {
                // Do nothing
            }
        }
    }

    /**
     * Obtenir la fentre en plein-cran.
     */
    public Window getFullScreen() {
        return gd.getFullScreenWindow();
    }
    
    /**
     * Obtenir le GraphicsDevice par dfaut de la fentre.
     */
    /*public GraphicsDevice getGraphicsDevice(){
        return gd;
    }*/

    /**
     * Arrte le mode plein-cran pour revenir en mode fentr ...
     */
    public void restoreFullScreen() {
        // On rcupre notre fentre en mode plein-cran
        Window window = getFullScreen();
        //
        if (window != null) window.dispose();
        // On arrte le plein-cran.
        gd.setFullScreenWindow(null);
    }
}